<?php
/**
 * Disable/enable annoying or necessary
 * runtime configuration settings
 */
ini_set('magic_quotes_sybase',false);
ini_set('magic_quotes_runtime',false);

date_default_timezone_set('UTC');

define('APP_DIR', dirname(__FILE__));

if (file_exists(APP_DIR . "/local.paths.php"))
	include (APP_DIR . "/local.paths.php");
else
	define('HIPPO_DIR', APP_DIR . '/opt/hippo');

/* Make use of the HiPPO library */
include(HIPPO_DIR . '/hippo.php');

/* 
 * Reduce includes(), the core.php file contains
 * base classes used on every request:
 * 	ForgeConfig
 * 	ForgeController
 * 	ForgeModules
 * 	ForgeModule
 * 	ForgeUrl
 * 	ForgeUrlRewrite
 * 	ForgeTranslate
 */
include('core.php');

/**
 * The __autoload() function is used when PHP cannot
 * find a class definition in currently included files.
 * 
 * The system will first look for a matching class file
 * in the APP_DIR/lib directory, and then
 * the HIPPO_DIR/lib directory.  This allows for
 * developers to override the HiPPO library class behaviour by
 * adding a class file to the /lib directory without
 * having to modify HiPPO library classes
 */
function __autoload($class_name) {
  $class_file= strtolower(str_replace('Hippo','',$class_name)) . '.php';
  if (file_exists(APP_DIR . '/lib/' . $class_file))
		include_once(APP_DIR . '/lib/' . $class_file); 
  elseif (file_exists(HIPPO_DIR . '/lib/' . $class_file))
    include_once(HIPPO_DIR . '/lib/' . $class_file);
  else
    die('Attempt to include invalid class file: ' . $class_file);
}

/* 
 * Plugins are event listeners which react to 
 * certain events firing in the process of fulfilling
 * the request. This will load up all plugin files 
 * and call init() for each one.
 *
 * Modules are mini-applications which can handle
 * sets of requests from the controller.
 */
HippoPlugins::init_from_dir(APP_DIR . '/plugins/enabled');

/* Grab static system configuration settings */
if (file_exists(APP_DIR . "/local.config.php"))
	include(APP_DIR . "/local.config.php");
?>
