<?php
/**
 * Class representing a single forge user
 *
 * @ingroup	Records
 */
class ForgeUser extends HippoRecord {
	/**
	 * Constructor.
	 * 
	 * You must pass a HippoSqlConnection resource, and the internal resource
	 * pointer is assigned the parameter's value.
	 *
	 * Optionally, you can pass in a second argument containing the primary
	 * key value(s) to load from the DB into the instance members
	 *
	 * @return 	void
	 * @param		connection	HippoSqlConnection instance
	 * @param		keys				(optional) Primary key(s) will autoload the object
	 */
	public function __construct($connection= null, $keys=null) {
    $this->table_name= 'ForgeUser';

		/* Register all table fields */
		$this->fields['user_id']=      			new HippoRecordField('user_id');
		$this->fields['user_id']->is_key= TRUE;
		$this->fields['user_id']->is_sequence= TRUE;
    $this->fields['timezone']=  				new HippoRecordField('timezone');
    $this->fields['email']=         		new HippoRecordField('email', $type=PDO::PARAM_STR);
    $this->fields['password']=         	new HippoRecordField('password', $type=PDO::PARAM_STR);
    $this->fields['display_name']=      new HippoRecordField('display_name', $type=PDO::PARAM_STR);
    $this->fields['language']=    			new HippoRecordField('language', $type=PDO::PARAM_STR);
    $this->fields['created_on']=    		new HippoRecordField('created_on', $type=PDO::PARAM_STR);
		$this->fields['last_updated_on']= 	new HippoRecordField('last_updated_on', $text=PDO::PARAM_STR);
		$this->fields['last_logged_on']= 		new HippoRecordField('last_logged_on', $type=PDO::PARAM_STR);
		$this->fields['is_deleted']= 				new HippoRecordField('is_deleted');

		/* Tell the class which fields to update on key violations */
    $this->key_update_fields= array('display_name', 'language', 'timezone', 'last_updated_on', 'email');

		parent::__construct($connection, $keys);
  }

	/**
	 * Quick helper function to only update the login time
	 * when a user is logging in
	 *
	 * @return	void
	 */
	public function update_login_time() {
		$sql= "UPDATE ForgeUser SET last_logged_on=NOW() WHERE user_id = ?";
		$this->connection->add_param($this->user_id);
		$this->connection->query($sql);
	}
}
?>
