<?php
/**
 * The PeopleHandler is the handler which handles all actions
 * for the displaying the public profiles and stats on Forge users, 
 * the editing of profile information
 */
class PeopleHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= 'people';
		$this->handler_vars['CUR_NAV']= 'people';
		HippoEvents::listen('on_after_person_edit', array($this, 'on_after_person_edit'));
		parent::__construct();
	}

	/**
	 * Fires when a user's profile is edited.  We use this event to update the MediaWiki
	 * tables accordingly...
	 *
	 * @return	bool		Successful or not?
	 */
	public function on_after_person_edit() {
		/* Try to find a user record in the MW table and update necessary information */
		if (! empty($this->handler_vars['original_email'])) {
			$original_email= $this->handler_vars['original_email'];
			$new_email= $this->handler_vars['email'];
			$new_real_name= $this->handler_vars['display_name'];
			$new_munged_user_name= PeopleHandler::munge_mw_user_name($new_real_name);
			$sql= "UPDATE mw_user SET user_email = ?, user_name = ?, user_real_name = ? WHERE user_email = ?";
			$cnn= HippoDb::master();
			$cnn->add_param($new_email);
			$cnn->add_param($new_munged_user_name);
			$cnn->add_param($new_real_name);
			$cnn->add_param($original_email);
			return $cnn->query($sql);
		}		
	}

	/**
	 * Removes a forge user and all it's associated relationships
	 *
	 * @note Only an admin is allowed to execute this...
	 */
	public function delete_person() {
		if (FALSE !== $this->session->get('is_admin')) {
			if (! isset($this->handler_vars['id']))
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "denied.php");
			$user_id= $this->handler_vars['id'];
			$cnn= HippoDb::master();
			
			/* Do they have a MW account? */
			$sql_deletes= array();
			if (FALSE !== ($mw_user_id= $cnn->get_value("SELECT mw_user.user_id FROM mw_user INNER JOIN ForgeUser ON email = user_email WHERE ForgeUser.user_id = {$user_id}"))) {
				$sql_deletes= array(
						"DELETE FROM  mw_user_groups WHERE ug_user = {$mw_user_id}"
					, "DELETE FROM  mw_user_newtalk WHERE user_id = {$mw_user_id}"
					, "DELETE FROM  mw_archive WHERE ar_user = {$mw_user_id}"
					, "DELETE FROM  mw_revision WHERE rev_user = {$mw_user_id}"
					, "UPDATE mw_page SET page_latest = (SELECT MAX(rev_id) FROM mw_revision WHERE rev_page = mw_page.page_id)"
					, "DELETE FROM  mw_user WHERE user_id = {$mw_user_id}"
				);
			}

			/* Have they added any snippets? */
			if (FALSE !== ($tools= $cnn->get_rows("SELECT t.tool_id FROM Tools t WHERE t.added_by = {$user_id}"))) {
				foreach ($tools as $tool) {
					$tool_id= $tool['tool_id'];
					$sql_deletes= array_merge($sql_deletes, array(
							"DELETE FROM ItemTags WHERE item_type = 2 AND item_id = {$tool_id}"
						, "DELETE FROM Observers WHERE item_type = 2 AND item_id = {$tool_id}"
						, "DELETE FROM VoteSummary WHERE item_type = 2 AND item_id = {$tool_id}"
						, "DELETE FROM Votes WHERE item_type = 2 AND item_id = {$tool_id}"
						, "DELETE FROM Comments WHERE item_type = 2 AND item_id = {$tool_id}"
						, "DELETE FROM Snippet2SnippetCategory WHERE snippet = {$tool_id}"
						, "DELETE FROM Snippet WHERE snippet_id = {$tool_id}"
						, "DELETE FROM CodeSnippets WHERE tool_id = {$tool_id}"
						, "DELETE FROM Tools WHERE tool_id = {$tool_id}"
						)
					);
				}
			}

			/* Have they added any projects? */
			if (FALSE !== ($projects= $cnn->get_rows("SELECT p.project_id FROM Project p WHERE p.added_by = {$user_id}"))) {
				foreach ($projects as $project) {
					$project_id= $project['project_id'];
					$sql_deletes= array_merge($sql_deletes, array(
							"DELETE FROM ItemTags WHERE item_type = 1 and item_id = {$project_id}"
						, "DELETE FROM Observers WHERE item_type = 1 AND item_id = {$project_id}"
						, "DELETE FROM ProjectText WHERE project = {$project_id}"
						, "DELETE FROM Project2ProjectCategory WHERE project = {$project_id}"
						, "DELETE FROM Project2Architecture WHERE project = {$project_id}"
						, "DELETE FROM ForgeUser2Project WHERE project = {$project_id}"
						, "DELETE FROM ProjectMember WHERE project = {$project_id}"
						, "DELETE FROM ProjectMemberReview WHERE project = {$project_id}"
						, "DELETE FROM VoteSummary WHERE item_type = 1 AND item_id = {$project_id}"
						, "DELETE FROM Votes WHERE item_type = 1 AND item_id = {$project_id}"
						, "DELETE FROM Comments WHERE item_type = 1 AND item_id = {$project_id}"
						, "DELETE FROM Project WHERE project_id = {$project_id}"
						)
					);
				}
			}

			$cnn->start_transaction();

			$sql_deletes= array_merge($sql_deletes, array(
				"DELETE FROM ItemTags WHERE item_type = 4 AND item_id = ?"
			, "DELETE FROM Observers WHERE forge_user = ?"
			, "DELETE FROM ForgeUser2Project WHERE forge_user = ?"
			, "DELETE FROM ProjectMember WHERE member = ?"
			, "DELETE FROM ProjectMemberReview WHERE member = ?"
			, "DELETE FROM Votes WHERE forge_user = ?"
			, "DELETE FROM Comments WHERE added_by = ?"
			, "DELETE FROM Contributor WHERE forge_user = ?"
			, "DELETE FROM ForgeUserProfile WHERE forge_user = ?"
			, "DELETE FROM ForgeUser WHERE user_id = ?"
			));
			foreach ($sql_deletes as $sql) {
				if (strpos($sql, '?') !== FALSE)
					$cnn->add_param($user_id);
				if (! $cnn->query($sql)) {
					$cnn->rollback();
					$this->session->set('message', "DELETE FAILED for {$sql}: {$cnn->get_last_error_message()}");
					HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/person.php?id={$user_id}");
				}
				else {
					$cnn->reset();
				}
			}
			$this->session->set('message',  "DELETE successful");
			$cnn->commit();
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/");
		}
		else
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "denied.php");
	}

	/**
	 * Helper function to convert display names to what MW uses internally as usernames
	 *
	 * @return	string		Munged name
	 * @param		name			Name to munge
	 */
	public static function munge_mw_user_name($name) {
		/** Based from MediaWiki's /include/User.php -> User::getCanonicalName() */
		$name= ucfirst($name);

		/**
		 * Convert things like &eacute; &#257; or &#x3017; into real text...
		 */
		include_once("../w/includes/Sanitizer.php");
		$name= Sanitizer::decodeCharReferences( $name );
		return $name;

	}

	/**
	 * Displays the login form
	 * @return void
	 * @param		show_captcha		Show a CAPTCHA?
	 */
	public function display_login($show_captcha= FALSE) {
		/* Fix for Bug #34956  	Login redirection vulnerability */
		if (isset($this->handler_vars['url_redirect']))
			$url_redirect= $this->handler_vars['url_redirect'];
		else {
			if (isset($_SERVER['HTTP_REFERER']))
				$url_redirect= $_SERVER['HTTP_REFERER'];
			else
				$url_redirect= '';
		}

		$url_redirect_pieces= parse_url($url_redirect);
		if (isset($url_redirect_pieces['host']) &&
			! in_array($url_redirect_pieces['host'], AppConfig::get_config('VALID_REDIRECT_HOSTS')))
			$url_redirect= '';
		if ($url_redirect == '')
			$url_redirect= AppConfig::get_config('DIR_HTTP');
		$this->handler_vars['url_redirect']= $url_redirect;
		$this->handler_vars['page_title']= 'People:: Login';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'login');

		if ($show_captcha)
			$this->handler_vars['captcha_html']= ForgeCaptcha::get_html();

		$this->display('login');
		exit;
	}

	/**
	 * Logs out the forge user and redirects to a home page
	 *
	 * @return void
	 */
	public function logout() {
		$this->session->destroy();
		/* 
		 * We must also destroy the MW cookie here as well 
		 * This taken from w/includes/User.php :(
		 */
		setcookie( 'mysqlforge_mw_session', '', time() - 3600, AppConfig::get_config('COOKIE_PATH'), AppConfig::get_config('COOKIE_DOMAIN'));
		setcookie( 'mysqlforge_mw_UserID', '', time() - 3600, AppConfig::get_config('COOKIE_PATH'), AppConfig::get_config('COOKIE_DOMAIN'));
		setcookie( 'mysqlforge_mw_UserName', '', time() - 3600, AppConfig::get_config('COOKIE_PATH'), AppConfig::get_config('COOKIE_DOMAIN'));
		setcookie( 'mysqlforge_mw_Token', '', time() - 3600, AppConfig::get_config('COOKIE_PATH'), AppConfig::get_config('COOKIE_DOMAIN'));
		$ts_now = gmdate('YmdHis', time()); // emulates wfTimestampNow()
		setcookie( 'mysqlforge_mw_LoggedOut', $tsnow, time() + 86400, AppConfig::get_config('COOKIE_PATH'), AppConfig::get_config('COOKIE_DOMAIN'));

		HippoEvents::notify('on_after_person_logout', $this);
		HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP'));
  }

	/**
	 * Gets local forge user ID from email address or FALSE if not found
	 *
	 * @return	mixed		Local user ID or FALSE if not found
	 * @param		email		Email of user to check for user ID
	 */
	private function get_local_user_id_from_email($email) {
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT user_id
FROM ForgeUser
ENDOFSQL;
		$reader->email= $email;
		if (FALSE !== ($forge_user= $reader->get_one()))
			return $forge_user['user_id'];
		return FALSE;
	}

	/**
	 * Gets local mediawiki user ID from email address or FALSE if not found
	 *
	 * @return	mixed		Local mediawiki user ID or FALSE if not found
	 * @param		email		Email of user to check for user ID
	 */
	private function get_local_wiki_user_id_from_email($email) {
		$reader= new MwUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT user_id
FROM mw_user
ENDOFSQL;
		$reader->user_email= $email;
		if (FALSE !== ($mw_user= $reader->get_one()))
			return $mw_user['user_id'];
		return FALSE;
	}

	/**
	 * Adds a local Forge User account based on information from the
	 * central API server.
	 *
	 * @return	mixed					user_id of new local account or FALSE
	 * @param		central_user	Associative array of central user account info
	 */
	private function create_local_user_from_central_user($central_user) {
		$connection= HippoDb::master();
		$forge_user= new ForgeUser($connection);
		$forge_user->email= $central_user['email'];;
		$forge_user->display_name= $central_user['first_name'] . ' ' . $central_user['last_name'];
		$forge_user->password= 'BOGUS-CENTRAL'; /* Just to mark it, since non NULLABLE field */
		$forge_user->created_on= date('Y-m-d H:i:s');
		$forge_user->last_updated_on= date('Y-m-d H:i:s');
		$forge_user->last_logged_on= date('Y-m-d H:i:s');
		$forge_user->is_deleted= "0"; /** @todo	If I change this to $forge_user->is_deleted= 0; it doesn't work... looks like a Bug in PHP... */
		$forge_user->created_on= date('Y-m-d H:i:s');
		$forge_user->timezone= 25; /* UTC */
		$forge_user->language= 'en'; /* English */
		$forge_user->fields['user_id']->can_be_null= TRUE;

		$connection->start_transaction();
		if (! $forge_user->save()) {
			$this->errors= $forge_user->errors;
			$connection->rollback();
			return FALSE;
		}
		else {
			$forge_user_profile= new ForgeUserProfile($connection);
			$forge_user_profile->forge_user= $forge_user->user_id;
			$forge_user_profile->first_name= $central_user['first_name'];
			$forge_user_profile->last_name= $central_user['last_name'];
			$forge_user_profile->url= NULL;
			$forge_user_profile->address= $central_user['address'];
			$forge_user_profile->city= $central_user['city'];
			$forge_user_profile->province= $central_user['state'];
			$forge_user_profile->country= $central_user['country'];
			$forge_user_profile->current_title= NULL;
			$forge_user_profile->current_company= $central_user['company'];
			$forge_user_profile->bio= NULL;
			if (! $forge_user_profile->save()) {
				$this->errors= $forge_user_profile->errors;
				$connection->rollback();
				return FALSE;
			}
			else {
				$connection->commit();
				HippoEvents::notify('on_after_person_add', $this); 
				$user_id= $forge_user->user_id;
				return $user_id;
			}
		}
	}

	/**
	 * Adds a local Forge User account based on information from the
	 * Mediawiki database user
	 *
	 * @return	mixed					user_id of new local wiki account or FALSE
	 * @param		forge_user_id	Local forge user ID
	 */
	private function create_local_forge_user_from_local_wiki_user($wiki_user_id) {
		$reader= new MwUserReader(HippoDb::master());
		$reader->user_id= $wiki_user_id;
		if (FALSE === ($wiki_user= $reader->get_one()))
			return FALSE;

		$connection= HippoDb::master();
		$forge_user= new ForgeUser($connection);
		$forge_user->email= $wiki_user['user_email'];
		/* Draw and quarter the real name to get a first and last name...*/
		$name_parts= explode(' ', $wiki_user['user_real_name']);
		if (count($name_parts) == 2) {
			$first_name= $name_parts[0];
			$last_name= $name_parts[1];
		}
		else {
			$first_name= $name_parts[0];
			$last_name= '';
		}
		$forge_user->display_name= $wiki_user['user_name'];
		$forge_user->password=  md5($this->handler_vars['password']); /* We know at this point the password supplied is indeed correct, so store it... */
		$forge_user->created_on= date('Y-m-d H:i:s');
		$forge_user->last_updated_on= date('Y-m-d H:i:s');
		$forge_user->last_logged_on= date('Y-m-d H:i:s');
		$forge_user->is_deleted= "0"; /** @todo	If I change this to $forge_user->is_deleted= 0; it doesn't work... looks like a Bug in PHP... */
		$forge_user->created_on= date('Y-m-d H:i:s');
		$forge_user->timezone= 25; /* UTC */
		$forge_user->language= 'en'; /* English */
		$forge_user->fields['user_id']->can_be_null= TRUE;

		$connection->start_transaction();
		if (! $forge_user->save()) {
			$this->errors= $forge_user->errors;
			$connection->rollback();
			return FALSE;
		}
		else {
			$forge_user_profile= new ForgeUserProfile($connection);
			$forge_user_profile->forge_user= $forge_user->user_id;
			$forge_user_profile->first_name= $first_name;
			$forge_user_profile->last_name= $last_name;
			$forge_user_profile->url= NULL;
			$forge_user_profile->address= NULL;
			$forge_user_profile->city= NULL;
			$forge_user_profile->province=  NULL;
			$forge_user_profile->country= 'US'; /* GEOIP? */
			$forge_user_profile->current_title= NULL;
			$forge_user_profile->current_company= NULL;
			$forge_user_profile->bio= NULL;
			if (! $forge_user_profile->save()) {
				$this->errors= $forge_user_profile->errors;
				$connection->rollback();
				return FALSE;
			}
			else {
				$connection->commit();
				HippoEvents::notify('on_after_person_add', $this); 
				$user_id= $forge_user->user_id;
				return $user_id;
			}
		}
	}

	/**
	 * Adds a local Forge User account based on information from the
	 * central API server.
	 *
	 * @return	mixed					user_id of new local wiki account or FALSE
	 * @param		forge_user_id	Local forge user ID
	 */
	private function create_local_wiki_user_from_local_forge_user($forge_user_id) {
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT email, display_name
FROM ForgeUser 
ENDOFSQL;
		$reader->user_id= $forge_user_id;
		if (FALSE === ($forge_user= $reader->get_one()))
			return FALSE;

		/* Silly MediaWiki...*/
		$user_options=<<<ENDOFOPTIONS
quickbar=1
underline=2
cols=80
rows=25
searchlimit=20
contextlines=5
contextchars=50
skin=mysqlforge
ENDOFOPTIONS;
		$munged_user_name= PeopleHandler::munge_mw_user_name($forge_user['display_name']);
		$sql=<<<ENDOFSQL
INSERT INTO mw_user (user_id, user_email, user_name, user_real_name, user_options) VALUES (NULL, ?, ?, ?, ?)
ON DUPLICATE KEY UPDATE user_name = ?, user_real_name = ?, user_options = ?
ENDOFSQL;
		$cnn= HippoDb::master();
		$cnn->add_param($forge_user['email']);
		$cnn->add_param($munged_user_name);
		$cnn->add_param($forge_user['display_name']);
		$cnn->add_param($user_options);
		$cnn->add_param($munged_user_name);
		$cnn->add_param($forge_user['display_name']);
		$cnn->add_param($user_options);
		if ($cnn->query($sql))
			return $cnn->last_insert_id();
		return FALSE;
	}

	/**
	 * Verify credentials against a local forge user account
	 *
	 * @return	bool					Checks out?
	 * @param		forge_user_id	Local forge user ID
	 * @param		password			Password to check
	 */
	private function passes_local_authentication($forge_user_id, $password) {
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT password FROM ForgeUser 
ENDOFSQL;
		$reader->user_id= $forge_user_id;
		if (FALSE === ($forge_user= $reader->get_one()))
			return FALSE;
		return ($forge_user['password'] == md5($password));
	}

	/**
	 * Verify credentials against a local wiki user account
	 *
	 * @return	bool					Checks out?
	 * @param		wiki_user_id	Local wiki user ID
	 * @param		password			Password to check
	 */
	private function passes_wiki_authentication($wiki_user_id, $password) {
		$reader= new MWUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT user_password FROM mw_user
ENDOFSQL;
		$reader->user_id= $wiki_user_id;
		if (FALSE === ($wiki_user= $reader->get_one()))
			return FALSE;
		return ($wiki_user['user_password'] == md5($wiki_user_id . '-' . md5($password))); /* Yes, indeed this is the dumbest thing I have ever seen. */
	}

	/**
	 * Adds a central user record based on information in the local forge 
	 * user db.
	 *
	 * @return	bool					Was central user created?
	 * @param		forge_user_id	Local forge user ID
	 */
	private function create_central_user_from_local_forge_user($forge_user_id) {
		if (FALSE !== AppConfig::get_config('DEBUG')) {
			/* Grab some user info form the local forge account and create the central record */
			$reader= new ForgeUserReader(HippoDb::master());
			$reader->user_id= $forge_user_id;
			$local_forge_user= $reader->get_one();
			$user_info= array(
				'email'=>$local_forge_user['email']
				/* 
				 * We pass the unhashed password here since the 
				 * password has already been verified against the local DB... 
				 */
			, 'password'=>$this->handler_vars['password'] 
			, 'country'=>$local_forge_user['country']
			, 'first_name'=>$local_forge_user['first_name']
			, 'last_name'=>$local_forge_user['last_name']
			);
			if (FALSE !== $mysql_auth->user_create($user_info)) 
				return TRUE;
			return FALSE;
		}
		else
			return TRUE; /* In debug mode, assume went OK */
	}

	/**
	 * Adds a central user record based on information in the local user 
	 * user db.
	 *
	 * @return	bool					Was central user created?
	 * @param		wiki_user_id	Local wiki user ID
	 */
	private function create_central_user_from_local_wiki_user($wiki_user_id) {
		if (FALSE !== AppConfig::get_config('DEBUG')) {
			/* Grab some user info form the local forge account and create the central record */
			$reader= new MWUserReader(HippoDb::slave());
			$reader->select_sql=<<<ENDOFSQL
SELECT user_email, user_real_name FROM mw_user
ENDOFSQL;
			$reader->user_id= $wiki_user_id;
			$wiki_user= $reader->get_one();
			/* Draw and quarter the real name to get a first and last name...*/
			$name_parts= explode(' ', $wiki_user['user_real_name']);
			if (count($name_parts) == 2) {
				$first_name= $name_parts[0];
				$last_name= $name_parts[1];
			}
			else {
				$first_name= $name_parts[0];
				$last_name= '';
			}
			$user_info= array(
				'email'=>$wiki_user['user_email']
				/* 
				 * We pass the unhashed password here since the 
				 * password has already been verified against the local DB... 
				 */
			, 'password'=>$this->handler_vars['password'] 
			, 'country'=>'US' /* GeoIP? */
			, 'first_name'=>$first_name
			, 'last_name'=>$last_name
			);
			if (FALSE !== $mysql_auth->user_create($user_info))
				return TRUE;
			return FALSE;
		}
		else
			return TRUE; /* In debug mode, assume went OK */
	}

	/**
	 * Logs in the forge user
	 *
	 * There are a number of things we need to do here because of the MediaWiki
	 * login process and the centralized authentication and DRPL API
	 *
	 * @TODO Document the code branching.
	 *
	 * @return void
	 */
	public function login() {
		$password= null;
		$email= null;
		$url_redirect= '';
		$persist_session= "0";
		
		/* Init some local variables */
		if (isset($this->handler_vars['persist_session']))
			$persist_session= $this->handler_vars['persist_session'] == 'on' ? "1" : "0";
		if (isset($this->handler_vars['password']))
			$password = $this->handler_vars['password'];
		if (isset($this->handler_vars['email']))
			$email= $this->handler_vars['email'];
		if (isset($this->handler_vars['url_redirect'])) {
			$url_redirect= $this->handler_vars['url_redirect'];

			/* Fix for Bug #34956  	Login redirection vulnerability */
			$url_redirect_pieces= parse_url($url_redirect);
			if (isset($url_redirect_pieces['host']) &&
				! in_array($url_redirect_pieces['host'], AppConfig::get_config('VALID_REDIRECT_HOSTS')))
				$url_redirect= '';
		}
		else
			$url_redirect= '';

		/* Do some rudimentary sanity checks on post */
		if (empty($password))
			$this->errors['password']= "Please type a password...";
		if (empty($email))
			$this->errors['email']= "Email address is required...";
		if (FALSE === strpos($email, '@'))
			$this->errors['email']= 'It looks like you did not enter a valid email address.  Remember to log in with your email address, not a username';

		if (!empty($this->errors)) {
			/* Rudimentary errors occurred... */
			$this->display_login(); 
			exit;
		}
		if (AppConfig::get_config('DEBUG'))
			$authenticated= TRUE;
		else {
			/* Check with Central Auth API */
			$mysql_auth= new MySQLAuthAPI(AppConfig::get_config('MYSQL_API_SERVER'));
			$authenticated= $mysql_auth->user_authenticate($email, $password);
		}
		if ($authenticated) {
			/* 
			 * OK, the central system has this email address.
			 *
			 * First, we check to see if a local user record
			 * exists.  If so, we set the $user_id and continue.
			 * If not, we try to normalize by fetching the central
			 * user record and creating a new local forge account.
			 * If that succeeds, we attempt to normalize the MediaWiki
			 * user table as well, if a record with that email does not
			 * exist... (Yuck!)
			 */
			if (FALSE !== ($local_user_id= $this->get_local_user_id_from_email($email))) {
				/* 
				 * OK, there IS a local forge user record.  Set the 
				 * $user_id and continue with checks for MediaWiki
				 */
				$user_id= $local_user_id;
				if (FALSE !== ($local_wiki_user_id= $this->get_local_wiki_user_id_from_email($email))) {
					$wiki_user_id= $local_wiki_user_id;
				}
				else {
					/*
					 * Local user record in forge exists, but no MediaWiki
					 * user record.  Create a user record in MediaWiki.
					 */
					if (FALSE !== ($local_wiki_user_id= $this->create_local_wiki_user_from_local_forge_user($user_id))) {
						$wiki_user_id= $local_wiki_user_id;
					}
					else {
						/*
						 * Could not create local forge user account.  Error out
						 */
						$this->errors['email']= 'We could not create a local wiki account.';
						$this->display_login();
						exit;
					}
				}
			}
			else {
				if (AppConfig::get_config('DEBUG') == TRUE) {
					if (FALSE !== ($local_wiki_user_id= $this->get_local_wiki_user_id_from_email($email))) {
						/*
						 * OK, there is a local wiki account, but no central one.
						 * Before we create the central one from information based
						 * on the local wiki, let's verify the supplied password
						 * is correct so that we can send the unhashed password
						 * along to the central server
						 */
						if (FALSE !== $this->passes_wiki_authentication($local_wiki_user_id, $password)) {
							/*
							 * Add to the local forge user account table
							 */
							if (FALSE !== ($local_user_id= $this->create_local_forge_user_from_local_wiki_user($local_wiki_user_id))) {
								/* OK, user created on central DB...set up the variables needed for session creation below */
								$user_id= $local_user_id;
								$wiki_user_id= $local_wiki_user_id;
							}
							else {
								/* Failed to create a local forge user account */
								$this->errors['email']= 'We could not create a new local forge account from the local wiki information';
								$this->display_login();
								exit;
							}
						}
						else {
							/* Failed local wiki authentication.  Error out and retry */
							$this->errors['email']= 'Invalid user credentials supplied.';
							$this->display_login();
							exit;
						}
					}
					else {
						/* No such account in forge user OR wiki...  Error out and retry */
						$this->errors['email']= 'Invalid user credentials supplied.';
						$this->display_login();
						exit;
					}
				}
				else {
					/*
					 * There is NO local forge user record.  Create one
					 * based on the central user record.
					 */
					/* NOT in debug mode */
					if (FALSE !== ($central_user= $mysql_auth->user_get($email))) {
						/**
						 * Here, let's fix Bug #35679  	MySQL Forge allows unfiltered account creation.
						 *
						 * Apparently, the centralized system allows spammers to create accounts.  Although
						 * the Forge doesn't, here we create a new Forge account from the central system, so
						 * before doing that, let's CAPTCHA the creation to ensure a real person
						 */
						if (FALSE === AppConfig::get_config('DEBUG')) {
							if (!isset($this->handler_vars['recaptcha_response_field'])) {
								/* Redirect to a user login with a CAPTCHA */
								$this->display_login($show_captcha= TRUE);
								exit;
							}
							/* Verify they're human... */
							$response= $this->handler_vars['recaptcha_response_field'];
							$challenge= $this->handler_vars['recaptcha_challenge_field'];

							if ($challenge == "") {
								$this->errors['captcha']= 'Please type the words in the box above';
								$this->display_login($show_captcha= TRUE);
								exit;
							}

							if (TRUE !== ($result= ForgeCaptcha::check_captcha($challenge, $response))) {
								$this->errors['captcha']= ForgeCaptcha::get_readable_error($result);
								$this->display_login($show_captcha= TRUE);
								exit;
							}
						}
						if (FALSE !== ($local_user_id= $this->create_local_user_from_central_user($central_user))) {
							$user_id= $local_user_id;

							/*
							 * Here, we check if a MediaWiki user record exists already 
							 * for this email address.  If not, we add one.
							 */
							if (FALSE !== ($local_wiki_user_id= $this->create_local_wiki_user_from_local_forge_user($user_id))) {
								$wiki_user_id= $local_wiki_user_id;
							}
							else {
								/*
								 * Could not create local forge user account.  Error out
								 */
								$this->errors['email']= 'We could not create a local wiki account.';
								$this->display_login();
								exit;
							}
						}
						else {
							/*
							 * Could not create local forge user account.  Error out
							 */
							$this->errors['email']= 'We could not create a local account.';
							$this->display_login();
							exit;
						}
					}
					else {
						/*
						 * We got an error trying to fetch the central user record...
						 */
						$this->errors['email']= 'We could not retrieve the central user record';
						$this->display_login();
						exit;
					}
				}
			} /* End NO local forge account, but have central authentication */
		}
		else {
			/*
			 * Failed authentication from central auth API
			 */
			if (! $mysql_auth->has_errors()) {
				$this->errors['email']= 'Error communicating with central API server';
				$this->display_login();
				exit;
			}
			else {
				/*
				 * OK, let's check to see if we got invalid credentials or 
				 * if the account doesn't exist at all.
				 */
				$error= $mysql_auth->get_errors();
				$error_code= $error['code'];
				switch ($error_code) {
				case 0: /* Invalid Password */
					$this->errors['email']= 'Invalid user credentials supplied.';
					$this->display_login();
					exit;
				case -1: /* No account with that email */
					/*
					 * OK, here let's see if there is a local forge user account
					 * with that email address.  If so, let's try to add the 
					 * local information to the central DB.
					 */
					if (FALSE !== ($local_user_id= $this->get_local_user_id_from_email($email))) {
						/*
						 * OK, there is a local forge account, but no central one.
						 * Before we create the central one from information based
						 * on the local forge, let's verify the supplied password
						 * is correct so that we can send the unhashed password
						 * along to the central server
						 */
						if (FALSE !== $this->passes_local_authentication($local_user_id, $password)) {
							if (FALSE !== $this->create_central_user_from_local_forge_user($local_user_id)) {
								/* OK, user created on central DB...set up the variables needed for session creation below */
								$user_id= $local_user_id;
								$wiki_user_id= $local_wiki_user_id;
							}
							else {
								/* We failed to create a new central DB user */
								$this->errors['email']= 'We failed to create a new account on the central MySQL.com domain';
								$this->display_login();
								exit;
							}
						}
						else {
							/* Failed local authentication.  Error out and retry */
							$this->errors['email']= 'Invalid user credentials supplied.';
							$this->display_login();
							exit;
						}
					}
					else {
						/*
						 * Alright, no local forge user account.  Is there a local mediawiki
						 * account with that email?  If so, try adding to the local forge user
						 * account and to the central DB.
						 */
						if (FALSE !== ($local_wiki_user_id= $this->get_local_wiki_user_id_from_email($email))) {
							/*
							 * OK, there is a local wiki account, but no central one.
							 * Before we create the central one from information based
							 * on the local wiki, let's verify the supplied password
							 * is correct so that we can send the unhashed password
							 * along to the central server
							 */
							if (FALSE !== $this->passes_wiki_authentication($local_wiki_user_id, $password)) {
								/*
								 * Add to the local forge user account table
								 */
								if (FALSE !== ($local_user_id= $this->create_local_forge_user_from_local_wiki_user($local_wiki_user_id))) {
									if (FALSE !== $this->create_central_user_from_local_wiki_user($local_wiki_user_id)) {
										/* OK, user created on central DB...set up the variables needed for session creation below */
										$user_id= $local_user_id;
										$wiki_user_id= $local_wiki_user_id;
									}
									else {
										/* We failed to create a new central DB user */
										$this->errors['email']= 'We failed to create a new account on the central MySQL.com domain';
										$this->display_login();
										exit;
									}
								}
							}
							else {
								/* Failed to create a local forge user account */
								$this->errors['email']= 'We could not create a new local forge account from the local wiki information';
								$this->display_login();
								exit;
							}
						}
						else {
							/* Failed local authentication.  Error out and retry */
							$this->errors['email']= 'Invalid user credentials supplied.';
							$this->display_login();
							exit;
						}
					} /* END no local forge account */
					break; /* Still in switch! :) */
				default:
					$this->errors['email']= 'Error communicating with central API server';
					$this->display_login();
					exit;
				} /* END switch on error_code */
			}
		}

		/*
		 * We get here if all the authentication and auto-creation
		 * checks have been passed.  If we get here, the above code
		 * has guaranteed that we have two variables correctly
		 * populated: $user_id and $wiki_user_id.
		 */

		/* Update the local user's login time */
		$user= new ForgeUser(HippoDb::master(), $user_id);
		$user->update_login_time();

		/* 
		 * Set up the session.
		 *
		 * At this point, we should have the local forge user ID, the
		 * local wiki user ID, the display name and whether or not
		 * to persist the session.		 
		 */
		$this->session->set('user_id', $user_id);
		$this->session->set('is_admin', $user->is_admin);
		$this->session->set('wiki_user_id', $wiki_user_id);
		$this->session->set('persist_session', $persist_session);
		$this->session->set('display_name', $user->display_name);
		$this->session->set('last_login_time', date('Y-m-d H:i:s'));

		/* Handle DRPL checks for logged-in user */
		if (!AppConfig::get_config('DEBUG') && FALSE == ($contact_drpl_check= $mysql_auth->drpl_contact($email))) {
			/* Check for errors.  If none, save the DRPL failure in session */
			if (! $mysql_auth->has_errors()) 
				$this->session->set('drpl', 'restricted');
			else {
				/* 
				 * Check errors... 
				 * @TODO FINISH THIS.
				 * */
			}
		}
		else
			$this->session->set('drpl', 'allowed');

		HippoEvents::notify('on_after_person_login');
		HippoHttpResponse::redirect($url_redirect);
  }

	/**
	 * Displays a single person.
	 *
	 * @return void
	 */
	public function display_person() {
		if (! isset($this->handler_vars['id']))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTPS') . 'people/');

		$person_id= (int) $this->handler_vars['id'];
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->user_id= $person_id;
		if (! $person= $reader->get_one())
			$person= array();

		if (! empty($person)) {
			/* We need to escape output here...*/
			$person['bio']= TextCleaner::clean_html($person['bio']); /** @TODO Cache this */

			/* 
			 * Set the editable flag if the current session 
			 * user is the one who added the snippet
			 */
			if ($person['user_id'] == $this->session->get('user_id')) {
				$person['editable_by_user']= TRUE;

				/* 
				 * Grab the various lookup value lists for the 
				 * Edit Profile tab and add into the template
				 */
				$reader= new LanguageReader(HippoDb::slave(), $this->cache);
				$reader->order('name');
				$this->handler_vars['languages']= $reader->get_hash();

				$reader= new TimezoneReader(HippoDb::slave(), $this->cache);
				$reader->order('utc_offset_min');
				$this->handler_vars['timezones']= $reader->get_hash();

				$reader= new CountryReader(HippoDb::slave(), $this->cache);
				$reader->order('name');
				$this->handler_vars['countries']= $reader->get_hash();
			}

			HippoEvents::notify('on_before_display_person', array('handler'=>$this, 'person'=>$person));

			/* Grab the mediawiki pages this person has contributed or edited */
			$reader= new MWPageReader(HippoDb::slave());
			$reader->select_sql =<<<ENDOFSQL
SELECT SQL_NO_CACHE
	mwp.page_title
, mwp.page_touched
FROM ForgeUser fu
INNER JOIN mw_user mwu
	ON fu.email = mwu.user_email
INNER JOIN mw_revision mwr
	ON mwu.user_id = mwr.rev_user 
INNER JOIN mw_page mwp
	ON mwr.rev_page = mwp.page_id
ENDOFSQL;
			$reader->user_id= $person_id;
			$reader->groupby_sql= "\nGROUP BY mwp.page_title ";
			$reader->order('rev_timestamp', 'desc');
			if (! $contributed_mwpages= $reader->get())
				$contributed_mwpages= array();
			$num_contributed_mwpages= count($contributed_mwpages);
			$this->handler_vars['total_contributed_mwpages']= $num_contributed_mwpages;
			$person['contributed_mwpages']= $contributed_mwpages;

			/* Grab the contributed tools for this person...*/
			$reader= new ToolReader(HippoDb::slave());
			$reader->select_sql =<<<ENDOFSQL
SELECT SQL_NO_CACHE
	t.tool_id
, t.title
, t.added_on
, cs.program_language
, pl.name as program_language_desc
, vs.total_votes
, COALESCE(FORMAT(vs.total_vote_value / vs.total_votes, 2)) as rating
FROM Tools t
	LEFT JOIN VoteSummary vs 
		ON t.tool_id = vs.item_id 
		AND vs.item_type = 2
	INNER JOIN CodeSnippets cs
		ON t.tool_id = cs.tool_id
	INNER JOIN ProgramLanguage pl
		ON cs.program_language = pl.program_language_id
ENDOFSQL;
			$reader->added_by= $person_id;
			$reader->order('tool_id', 'desc');
			if (! $contributed_tools= $reader->get())
				$contributed_tools= array();
			$num_contributed_tools= count($contributed_tools);
			$this->handler_vars['total_contributed_tools']= $num_contributed_tools;
			for ($i=0;$i<$num_contributed_tools;++$i)
				$contributed_tools[$i]['rating_width']= (16 * $contributed_tools[$i]['rating']);
			$person['contributed_tools']= $contributed_tools;

			/* Grab the contributed projects for this person...*/
			$reader= new ProjectReader(HippoDb::slave());
			$reader->select_sql=<<<ENDOFSQL
SELECT SQL_NO_CACHE
	p.project_id
, p.name
, pl.name as program_language_desc
, p.added_on
, vs.total_votes
, COALESCE(FORMAT(vs.total_vote_value / vs.total_votes, 2)) as rating
FROM Project p
	LEFT JOIN VoteSummary vs 
		ON p.project_id = vs.item_id 
		AND vs.item_type = 1
	INNER JOIN ProgramLanguage pl
		ON p.program_language = pl.program_language_id
ENDOFSQL;
			$reader->added_by= $person_id;
			$reader->order('project_id', 'desc');
			if (! $contributed_projects= $reader->get())
				$contributed_projects= array();
			$num_contributed_projects= count($contributed_projects);
			$this->handler_vars['total_contributed_projects']= $num_contributed_projects;
			for ($i=0;$i<$num_contributed_projects;++$i)
				$contributed_projects[$i]['rating_width']= (16 * $contributed_projects[$i]['rating']);
			$person['contributed_projects']= $contributed_projects;

			/* Grab the tags related to this user */
			$reader= new ItemTagReader(HippoDb::slave());
			$reader->item_type= 4; /* 4 === Forge User */
			$reader->item_id= $person_id;
			if (! $user_tags= $reader->get())
				$user_tags= array();
			$person['tags']= $user_tags;
			$this->handler_vars['total_tags']= count($user_tags);
			$person['taglink']= AppConfig::get_config('DIR_HTTP') . 'people/search.php?t=tag&k=';

			$this->handler_vars['page_title']= 'People :: ' . htmlentities($person['display_name']);
			$this->handler_vars['breadcrumbs'][]= array('link'=>'people/', 'title'=>'dashboard');
			$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>htmlentities($person['display_name']));
		}

		/* We need to add some javascript files and CSS files for tab panel display */
		$this->handler_vars['scripts'][]= "yui-ext/adapter/yui/yui-utilities.js";
		$this->handler_vars['scripts'][]= "yui-ext/adapter/yui/ext-yui-adapter.js";
		$this->handler_vars['scripts'][]= "yui-ext/ext-core.js";
		$this->handler_vars['scripts'][]= "yui-ext/package/util.js";
		$this->handler_vars['scripts'][]= "yui-ext/package/tabs/tabs.js";

		$this->handler_vars['stylesheets'][]= "../js/yui-ext/resources/css/ext-all.css";
		$this->handler_vars['stylesheets'][]= "../js/yui-ext/resources/css/ytheme-gray.css";

		$this->handler_vars['person']= $person;

		$this->display('person');
	}

	/**
	 * Adds a new or edits an existing forge member
	 *
	 * If the edit succeeds or not, a redirection occurs to 
	 * the person's public profile page, with errors showing or not.
	 *
	 * @return void
	 */
	public function edit_person() {
		$new_user= (!	$this->session->user_id  || ! isset($this->handler_vars['user_id']));

		/*
		 * Country is now required, as it is important for central auth API
		 * and for DRPL checks.  Quickly check here and error out if missing
		 */
		if (! isset($this->handler_vars['country'])
			|| empty($this->handler_vars['country'])) {
			$this->errors['country']= 'Please select your home country';
			if ($new_user) {
				$this->display_add_person();
				exit;
			}
			else
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/person.php?id={$this->handler_vars['user_id']}");
		}

		if ($new_user) {
			/* 
			 * Before trying any fancy stuff with the central authentication
			 * API, let's first do some rudimentary checks first and 
			 * examine the CAPTCHA.
			 */
			if (FALSE === AppConfig::get_config('DEBUG')) {
				/* Verify they're human... */
				$response= $this->handler_vars['recaptcha_response_field'];
				$challenge= $this->handler_vars['recaptcha_challenge_field'];

				if ($challenge == "") {
					$this->errors['captcha']= 'Please type the words in the box above';
					$this->display_add_person();
					exit;
				}

				if (TRUE !== ($result= ForgeCaptcha::check_captcha($challenge, $response))) {
					$this->errors['captcha']= ForgeCaptcha::get_readable_error($result);
					$this->display_add_person();
					exit;
				}
			}

			/* Verify passwords match */
			$pass= $this->handler_vars['password'];
			$confirm_pass= $this->handler_vars['confirm_password'];
			if (trim($pass) == "") {
				$this->errors['password']= "Password is required";
				$this->display_add_person();
				return;
			}
			if ($pass !== $confirm_pass) {
				$this->errors['confirm_password']= "Password and confirmation password did not match";
				$this->display_add_person();
				return;
			}

			/*
			 * Here, we need to check if an account with that email already exists
			 * in the central system.  If it does, we need to redirect the user
			 * to a screen which shows the different fields between what they
			 * just entered and the central account information
			 *
			 * If a central account does NOT exist, we first need to attempt to 
			 * create an account on the central system before trying to create
			 * a local forge account.  If central account creation fails, we must error
			 * out and not create a local forge account.
			 */
			if (FALSE === AppConfig::get_config('DEBUG')) {
				$mysql_auth= new MySQLAuthAPI(AppConfig::get_config('MYSQL_API_SERVER'));
				if (! FALSE === ($central_user= $mysql_auth->user_get($this->handler_vars['email']))) {
					$user_info= array(
						'country'=>$this->handler_vars['country']
						, 'email'=>$this->handler_vars['email']
						, 'state'=>$this->handler_vars['state']
						, 'city'=>$this->handler_vars['city']
						, 'address'=>$this->handler_vars['address']
						, 'company'=>$this->handler_vars['current_employer']
						, 'first_name'=>$this->handler_vars['first_name']
						, 'last_name'=>$this->handler_vars['last_name']
						);
					if (! FALSE === ($mysql_auth->user_diff($user_info))) {
						/* Redirect to a sync-field screen */
						$this->display_synchronize_person($central_user);
						exit;
					}
				}
				else {
					/*  Central account does not exist.  Attempt to create it. */
					$new_central_user= array(
						'email'=>$this->handler_vars['email']
					, 'password'=>$this->handler_vars['password']
					, 'first_name'=>$this->handler_vars['first_name']
					, 'last_name'=>$this->handler_vars['last_name']
					, 'city'=>$this->handler_vars['city']
					, 'state'=>$this->handler_vars['province']
					, 'country'=>$this->handler_vars['country']
					, 'company'=>$this->handler_vars['current_company']
					);
					if (FALSE === $mysql_auth->user_create($new_central_user)) {
						/* Central account creation failed.  We must error out now. */
						$error= $mysql_auth->get_errors();
						$this->errors['email']= $error['message'];
						$this->display_add_person();
						exit;
					}
				} /* END not DEBUG */
			}
		}

		$connection= HippoDb::master();
		$forge_user= new ForgeUser($connection);

		/* Quick hack to stop spammers... */
		$ip_address_spammers= array('85.141.65.149');
		$user_ip= sprintf("%u", ip2long($_SERVER['REMOTE_ADDR']));
		if (in_array($user_ip, $ip_address_spammers))
			exit;

		if (! $new_user) {
			$forge_user->user_id= (int) $this->handler_vars['user_id'];
			$forge_user->fields['password']->can_be_null= TRUE;
			$forge_user->fields['password']->value=''; /** @todo Better/elegant... */
		}
		else
			$forge_user->fields['user_id']->can_be_null= TRUE;

		$forge_user->last_updated_on= date('Y-m-d H:i:s');
		$forge_user->last_logged_on= date('Y-m-d H:i:s');
		$forge_user->is_deleted= "0"; /** @todo	If I change this to $forge_user->is_deleted= 0; it doesn't work... looks like a Bug in PHP... */
		$forge_user->created_on= date('Y-m-d H:i:s');

		$forge_user->display_name= $this->handler_vars['display_name'];
		$forge_user->email= $this->handler_vars['email'];
		$forge_user->timezone= (int) $this->handler_vars['timezone'];
		$forge_user->language= $this->handler_vars['language'];

		if ($new_user) {
			$pass= md5($pass);
			$forge_user->password= $pass;

			/*
			 * Before we can proceed, we need to ensure that if the email address 
			 * was changed, we verify that no other user account has that email 
			 * address...
			 */
			$cnn= HippoDb::master();
			$cnn->add_param($this->handler_vars['email']);
			$email_exists= (int) $cnn->get_value("SELECT COUNT(*) FROM ForgeUser WHERE email = ?");
			if ($email_exists != 0) {
				$this->errors['email']= 'Sorry, an account with that email address already exists.';
				$this->display_add_person();
				return;
			}
		}
		else {
			/* 
			 * To be sure we sync up with the MW database entry, 
			 * we grab the existing user data before updating the DB
			 */
			$this->handler_vars['original_email']= HippoDb::master()->get_value("SELECT email FROM ForgeUser WHERE user_id = {$forge_user->user_id}");

			/*
			 * Before we can proceed, we need to ensure that if the email address 
			 * was changed, we verify that no other user account has that email 
			 * address...
			 */
			if ($this->handler_vars['original_email'] != $this->handler_vars['email']) {
				$cnn= HippoDb::master();
				$cnn->add_param($this->handler_vars['email']);
				$email_exists= (int) $cnn->get_value("SELECT COUNT(*) FROM ForgeUser WHERE email = ? AND user_id != {$forge_user->user_id}");
				if ($email_exists != 0) {
					$this->errors['email']= 'Sorry, an account with that email address already exists.';
					HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/person.php?id={$forge_user->user_id}");
				}
			}
		}

		/* Check whether the URL is good or not... */
		if (isset($this->handler_vars['url'])
			&& $this->handler_vars['url'] != '') {
			include_once(APP_DIR . '/opt/pear-validate/Validate.php');
			if (! Validate::uri($this->handler_vars['url'])) {
				if ($new_user) {
					$this->errors['url']= "Invalid URL entered.";
					$this->display_add_person();
					return;
				}
				else {
					$this->errors['url']= "Invalid URL entered.";
					HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/person.php?id={$forge_user->user_id}");
				}
			}
		}

		$email= $this->handler_vars['email'];

		$connection->start_transaction();
		if (! $forge_user->save()) {
			$this->errors= $forge_user->errors;
			$connection->rollback();
		}
		else {
			$forge_user_profile= new ForgeUserProfile($connection);
			$forge_user_profile->forge_user= $forge_user->user_id;
			$forge_user_profile->first_name= $this->handler_vars['first_name'];
			$forge_user_profile->last_name= $this->handler_vars['last_name'];
			$forge_user_profile->url= $this->handler_vars['url'];
			$forge_user_profile->address= $this->handler_vars['address'];
			$forge_user_profile->city= $this->handler_vars['city'];
			$forge_user_profile->province= $this->handler_vars['province'];
			$forge_user_profile->country= $this->handler_vars['country'];
			$forge_user_profile->current_title= $this->handler_vars['current_title'];
			$forge_user_profile->current_company= $this->handler_vars['current_company'];
			//$forge_user_profile->bio= $this->handler_vars['bio'];
			if (! $forge_user_profile->save()) {
				$this->errors= $forge_user_profile->errors;
				$connection->rollback();}
			else {
				$connection->commit();
				if ($new_user) {
					HippoEvents::notify('on_after_person_add', $this); 

					/* Perform login */
					$forge_user->update_login_time();
					$this->session->set('user_id', $forge_user->user_id);
					$this->session->set('display_name', $forge_user->display_name);
					$this->session->set('message', "New account creation successful");
					$this->session->set('persist_session', "0");
					$this->session->set('last_login_time', date('Y-m-d H:i:s'));

					/* Handle DRPL checks for logged-in user */
					if (! isset($mysql_auth) && AppConfig::get_config('DEBUG') == FALSE)
						$mysql_auth= new MySQLAuthAPI(AppConfig::get_config('MYSQL_API_SERVER'));

					if (!AppConfig::get_config('DEBUG') && FALSE == ($contact_drpl_check= $mysql_auth->drpl_contact($email))) {
						/* Check for errors.  If none, save the DRPL failure in session */
						if (! $mysql_auth->has_errors()) 
							$this->session->set('drpl','restricted');
						else {
							/* 
							 * Check errors... 
							 * @TODO FINISH THIS.
							 * */
						}
					}
					else
						$this->session->set('drpl', 'allowed');

					HippoEvents::notify('on_after_person_login');
				}
				else {
					HippoEvents::notify('on_after_person_edit', $this); /* Among other things, the MW DB is updated here...*/
					$this->session->set('user_id', $forge_user->user_id);
					$this->session->set('display_name', $forge_user->display_name);
					$this->session->set('message', "Save successful");
					$this->session->set('persist_session', "0");
					$this->session->set('last_login_time', date('Y-m-d H:i:s'));


					/* Handle DRPL checks for logged-in user */
					if (! isset($mysql_auth) && AppConfig::get_config('DEBUG') == FALSE)
						$mysql_auth= new MySQLAuthAPI(AppConfig::get_config('MYSQL_API_SERVER'));

					if (!AppConfig::get_config('DEBUG') && FALSE == ($contact_drpl_check= $mysql_auth->drpl_contact($email))) {
						/* Check for errors.  If none, save the DRPL failure in session */
						if (! $mysql_auth->has_errors()) 
							$this->session->set('drpl',  'restricted');
						else {
							/* 
							 * Check errors... 
							 * @TODO FINISH THIS.
							 * */
						}
					}
					else
						$this->session->set('drpl', 'allowed');
				}
				/* Redirect here to prevent double-saves on refresh */
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/person.php?id={$forge_user->user_id}");
			}
		}
		/* Save was not successful */
		if ($new_user) 
			$this->display_add_person();
		else
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "people/person.php?id={$forge_user->user_id}");
	}

	/**
	 * Displays the form for synchronizing account information
	 * between the central and local account records
	 *
	 * @return 	void
	 * @param		central_user	Associative array of account data to sync with
	 */
	public function display_synchronize_person($central_user) {
		$field_labels= array(
			'country'=>'Your home country'
		, 'address'=>'Your home address'
		, 'city'=>'Your home city'
		, 'state'=>'Your home state or province'
		, 'first_name'=>'Your first name'
		, 'last_name'=>'Your last name'
		, 'current_employer'=>'Your current employer'
		);

		/* 
		 * Loop through the central user data and see 
		 * which of the posted field values are mismatched, if any
		 */
		$diffs= array();
		foreach ($this->handler_vars as $key=>$posted_value) {
			if (isset($central_user[$key]) 
				&& in_array($key, array_keys($field_labels)) /* We don't care about user_id, last_updated_on, email...*/
				&& $central_user[$key] != $posted_value)
				$diffs[]= array(
					'name'=>$key
				, 'label'=>$field_labels[$key]
				, 'local_value'=>$posted_value
				, 'central_value'=>$central_user[$key]);
		}
		$this->handler_vars['diffs']= $diffs;

		$this->handler_vars['page_title']= 'People :: Synchronize Account Information';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'people/', 'title'=>'dashboard');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'Synchronize Account Information');

		$this->display('sync');
	}

	/**
	 * Displays the form for registering a new forge user 
	 * @return void
	 */
	public function display_add_person() {
		/* 
		 * Grab the various lookup value lists for the 
		 * new account form and add into the template
		 */
		$reader= new LanguageReader(HippoDb::slave(), $this->cache);
		$reader->order('name');
		$this->handler_vars['languages']= $reader->get_hash();

		$reader= new TimezoneReader(HippoDb::slave(), $this->cache);
		$reader->order('utc_offset_min');
		$this->handler_vars['timezones']= $reader->get_hash();

		$reader= new CountryReader(HippoDb::slave(), $this->cache);
		$reader->order('name');
		$this->handler_vars['countries']= $reader->get_hash();

		$person= new ForgeUser(HippoDb::master());
		$person->fill_field_values($this->handler_vars);

		$person_profile= new ForgeUserProfile(HippoDb::master());
		$person_profile->fill_field_values($this->handler_vars);

		$this->handler_vars['page_title']= 'People :: Create Account';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'people/', 'title'=>'dashboard');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'Create Account');
		$this->handler_vars['person']= $person;
		$this->handler_vars['person_profile']= $person_profile;

		$this->handler_vars['captcha_html']= ForgeCaptcha::get_html();

		$this->display('add');
	}

	/**
	 * Displays the home page dashboard of the people repository
	 * @return void
	 * @todo Make more dynamic? 
	 */

	public function display_dashboard() {
		/* Grab a little info on the last 15 newest people (Forge accounts) */
		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->order('created_on', 'desc');
		/* Override default SELECT as we don't need all the info for the dashboard */
		$reader->select_sql =<<<ENDOFSQL
SELECT 
	fu.user_id
, fu.display_name
, fu.created_on
FROM ForgeUser fu
ENDOFSQL;
		if (! $newest_people= $reader->get(10))
			$newest_people= array();
		$this->handler_vars['total_people']= $reader->get_count();
		$this->handler_vars['newest_people']= $newest_people;

		/* Grab a little info on the top 10 rated people */
		/*
		 * Override default SELECT as we don't need all the info for the 
		 * dashboard and we want the ratings information too...
		 */
		$cnn= HippoDb::slave();
		$sql=<<<ENDOFSQL
SELECT 
	fu.user_id
, fu.display_name
, COUNT(*) as num_tools_added
FROM ForgeUser fu
INNER JOIN Tools t
ON fu.user_id = t.added_by
GROUP BY fu.user_id
ORDER BY num_tools_added DESC
LIMIT 10
ENDOFSQL;
		if (! $active_people= $cnn->get_rows($sql))
			$active_people= array();
		$this->handler_vars['active_people']= $active_people;

		/* Grab the tags related to any forge user */
		$reader= new ItemTagReader(HippoDb::slave());
		$reader->item_type= 4; /* 4 == Forge User */
		$reader->order('tag_text');
		if (! $user_tags= $reader->get())
			$user_tags= array();
		$this->handler_vars['user_tags']= $user_tags;
		$this->handler_vars['tag_link']= AppConfig::get_config('DIR_HTTP') . 'people/search.php?t=tag&k=';

		$this->handler_vars['page_title']= 'People :: Home';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'home');

		/* We need to add some javascript files for the accordian display */
		$this->handler_vars['scripts'][]= "scriptaculous/lib/prototype.js";
		$this->handler_vars['scripts'][]= "scriptaculous/src/effects.js";
		$this->handler_vars['scripts'][]= "scriptaculous/src/accordion.js";


		$this->display('dashboard');
	}

	/**
	 * Displays the search form or results of search
	 * @return void
	 */
	public function display_search() {
		$search_request= new PeopleSearchRequest();
		$search_request->records_per_page= 20;

		if (!empty($this->handler_vars['k'])) { /* Keywords... */
			$keywords= $this->handler_vars['k'];
			$look_in= $this->handler_vars['t'];
			switch ($look_in) {
			case 'tds': /* title and description... */
				$keywords= explode(',', $keywords);
				foreach ($keywords as $keyword) {
					$search_request->reader->display_name->like("%{$keyword}%", FALSE); /* FALSE == or condition */
					$search_request->reader->bio->like("%{$keyword}%", FALSE); /* FALSE == or condition */
				}
				break;
			case 'tag':
				$search_request->with_tags($keywords);
				break;
			}
			$search_request->records_per_page= 2000; /** @todo Figure out the LEFT JOIN issue with counts in the PeopleSearchRequest! */
		}

		if (! $search_results= $search_request->get())
			$search_results= array();

		$this->handler_vars['search_results']= $search_results;
		$this->handler_vars['search_request']= $search_request;

		$this->handler_vars['page_title']= 'People :: Search Results';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'people/', 'title'=>'people');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'search results');

		$this->display('search-results');
	}
}
?>
