<?php
/**
 * A specialized search request for forge members
 */
class PeopleSearchRequest extends SearchRequest {
	/**
	 * Constructor
	 */
	public function __construct() {
		
		$this->result_columns[] = array('name'=>'display_name', 'title'=>'Display Name', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'created_on', 'title'=>'Member Since', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'num_projects', 'title'=>'# Added<br />Projects', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'num_tools', 'title'=>'# Contributed<br />Tools', 'is_sortable'=>true);

		$this->sort_field= 'created_on';
		$this->sort_order= 'desc';

		$reader= new ForgeUserReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT 
	fu.user_id
, fu.display_name
, fu.created_on
, COUNT(DISTINCT p.project_id) AS num_projects
, COUNT(DISTINCT t.tool_id) AS num_tools
FROM ForgeUser fu
	LEFT JOIN ForgeUserProfile fup
		ON fu.user_id = fup.forge_user
	LEFT JOIN Tools t
		ON fu.user_id = t.added_by
	LEFT JOIN Project p
		ON fu.user_id = p.added_by
ENDOFSQL;
		$reader->groupby_sql= " GROUP BY fu.user_id";
		parent::__construct($reader);
	}

	/**
	 * Adds tag searching
	 *
	 * @return	void
	 * @params	tags		String of comma-separated tag phrases
	 */
	public function with_tags($tags) {
		/* First, grab a list of tag IDs for the supplied phrases */
		$reader= new TagReader(HippoDb::slave());
		$tags= explode(',', $tags);
		foreach ($tags as $tag)
			$reader->tag_text->equal_to($tag, FALSE); /* The FALSE adds an OR condition */
		if (! $tag_records= $reader->get()) 
			return FALSE;
		foreach ($tag_records as $tag_record)
			$tag_ids[]= $tag_record['tag_id'];
		$this->reader->select_sql.= " INNER JOIN Tag2ForgeUser t2fu ON fu.user_id = t2fu.forge_user AND t2fu.tag IN (" . implode(',', $tag_ids) . ") ";
	}
}
?>
