<?php
/**
 * The ProjectsHandler is the handler which handles all actions
 * for the displaying the public project information and comments
 * and for editing all project information
 */
class ProjectHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= 'projects';
		$this->handler_vars['CUR_NAV']= 'projects';
		parent::__construct();
	}

	/**
	 * Removes a project and all it's associated relationships
	 *
	 * @note Only an admin is allowed to execute this...
	 */
	public function delete_project() {
		if (FALSE !== $this->session->get('is_admin')) {
			if (! isset($this->handler_vars['id']))
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "denied.php");
			$project_id= $this->handler_vars['id'];
			$cnn= HippoDb::master();
			$cnn->start_transaction();

			$sql_deletes= array(
				"DELETE FROM ItemTags WHERE item_type = 1 AND item_id = ?"
			, "DELETE FROM Observers WHERE item_type = 1 AND item_id = ?"
			, "DELETE FROM ProjectText WHERE project = ?"
			, "DELETE FROM Project2ProjectCategory WHERE project = ?"
			, "DELETE FROM Project2Architecture WHERE project = ?"
			, "DELETE FROM ForgeUser2Project WHERE project = ?"
			, "DELETE FROM ProjectMember WHERE project = ?"
			, "DELETE FROM ProjectMemberReview WHERE project = ?"
			, "DELETE FROM VoteSummary WHERE item_type = 1 AND item_id = ?"
			, "DELETE FROM Votes WHERE item_type = 1 AND item_id = ?"
			, "DELETE FROM Comments WHERE item_type = 1 AND item_id = ?"
			, "DELETE FROM Project WHERE project_id = ?"
			);
			foreach ($sql_deletes as $sql) {
				$cnn->add_param($project_id);
				if (! $cnn->query($sql)) {
					$cnn->rollback();
					$this->session->set('message', "DELETE FAILED for {$sql}");
					HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$project_id}");
				}
				else {
					$cnn->reset();
				}
			}
			$this->session->set('message', "DELETE successful");
			$cnn->commit();
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "projects/");
		}
		else
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "denied.php");
	}

	/**
	 * Adds a new or edits an existing project.
	 *
	 * If the edit succeeds or not, a redirection occurs to 
	 * the project page, with errors showing or not.
	 *
	 * @return void
	 */
	public function edit_project() {
		if (FALSE === $this->session->get('user_id'))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . 'denied.php');

		$new_project= TRUE;
		if (isset($this->handler_vars['project_id'])) {
			$new_project= FALSE;
			$project_id= (int) $this->handler_vars['project_id'];
		}

		/* Check whether the URL is good or not... */
		if (isset($this->handler_vars['url'])
			&& $this->handler_vars['url'] != '') {
			include_once(APP_DIR . '/opt/pear-validate/Validate.php');
			if (! Validate::uri($this->handler_vars['url'])) {
				if ($new_project) {
					$this->errors['url']= "Invalid URL entered.";
					$this->display_add_project();
					return;
				}
				else {
					$this->errors['url']= "Invalid URL entered.";
					HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$project_id}");
				}
			}
		}

		$connection= HippoDb::master();
		$project= new Project($connection);

		if (isset($this->handler_vars['project_id'])) {
			$project->project_id= (int) $this->handler_vars['project_id'];
			/** 
			 * These columns are ignored on update...
			 *
			 * @todo Find a more elegant way of handling this... 
			 */
			$project->added_by= (int) $this->session->get('user_id');
			$project->added_on= date('Y-m-d H:i:s');
		}
		else {
			$project->fields['project_id']->can_be_null= TRUE;
			$project->added_by= (int) $this->session->get('user_id');
			$project->added_on= date('Y-m-d H:i:s');
		}

		$project->program_language= (int) $this->handler_vars['program_language'];
		$project->primary_status= (int) $this->handler_vars['primary_status'];
		$project->primary_audience= (int) $this->handler_vars['primary_audience'];
		$project->license= (int) $this->handler_vars['license'];
		$project->audience_knowledge_level= (int) $this->handler_vars['audience_knowledge_level'];
		$project->url= $this->handler_vars['url'];
		$project->name= $this->handler_vars['name'];
		$project->last_updated_by= (int) $this->session->get('user_id');
		$project->last_updated_on= date('Y-m-d H:i:s');
		$project->description= $this->handler_vars['description'];

		$connection->start_transaction();
		if (! $project->save()) {
			$this->errors= $project->errors;
			$connection->rollback();
		}
		else {
			$project_text= new ProjectText($connection);
			$project_text->project= $project->project_id;
			$project_text->name= $this->handler_vars['name'];
			$project_text->description= $this->handler_vars['description'];
			if (! $project_text->save()) {
				$this->errors= $project_text->errors;
				$connection->rollback();}
			else {
				$connection->commit();
				if (isset($this->handler_vars['project_id']))
					HippoEvents::notify('on_after_project_edit', $this);
				else
					HippoEvents::notify('on_after_project_add', $this);
				$this->session->set('message', "Save successful");
				/* Redirect here to prevent double-saves on refresh */
				HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$project->project_id}");
			}
		}
		if (isset($this->handler_vars['project_id'])) {
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$project->project_id}");
		}
		else
			$this->display_add_project();
	}

	/**
	 * Displays a single project 
	 * @return void
	 */
	public function display_project() {
		$project_id= (int) $this->handler_vars['id'];
		$reader= new ProjectReader(HippoDb::slave());
		$reader->project_id= $project_id;
		if (! $project= $reader->get_one())
			$project= array();

		/* Ensure we override the project values with posted ones */
		foreach ($this->handler_vars as $key=>$value)
			if (isset($project[$key]))
				$project[$key]= $value;

		if (! empty($project)) {
			/* 
			 * Set the editable flag if the current session 
			 * user is the one who added the project
			 */
			if ($project['added_by'] == $this->session->get('user_id')) {
				$project['editable_by_user']= TRUE;

				/* 
				 * Grab the various lookup value lists for the 
				 * edit Project tab and add into the template
				 */
				$reader= new ProgramLanguageReader(HippoDb::slave(), $this->cache);
				$this->handler_vars['program_languages']= $reader->get_hash();

				$reader= new KnowledgeLevelReader(HippoDb::slave(), $this->cache);
				$this->handler_vars['knowledge_levels']= $reader->get_hash();

				$reader= new LicenseReader(HippoDb::slave(), $this->cache);
				$this->handler_vars['licenses']= $reader->get_hash();

				$reader= new ProjectStatusReader(HippoDb::slave(), $this->cache);
				$this->handler_vars['project_statuses']= $reader->get_hash();

				$reader= new ProjectAudienceReader(HippoDb::slave(), $this->cache);
				$this->handler_vars['project_audiences']= $reader->get_hash();
			}

			/* Grab the number of observing (tracking) users */
			$reader= new ObserverReader(HippoDb::slave());
			$reader->item_type= 1; /* PROJECT */
			$reader->item_id = $project_id;
			$project['num_observers']= $reader->get_count();

			if (FALSE !== $this->session->get('user_id')) {
				$reader->forge_user= (int) $this->session->get('user_id');
				$project['observing']= ($reader->get_count() > 0);
			}
			else
				$project['observing']= FALSE;

			/* Grab the tags related to this project */
			$reader= new ItemTagReader(HippoDb::slave());
			$reader->item_id= $project_id;
			$reader->item_type= 1; /* 1 == Project */
			$reader->order('tag_text');
			if (! $project_tags= $reader->get())
				$project_tags= array();
			$project['tags']= $project_tags;
			$this->handler_vars['total_tags']= count($project_tags);
			$project['taglink']= AppConfig::get_config('DIR_HTTP') . 'projects/search.php?t=tag&k=';

			/* Grab the comments for the project too...*/
			$reader= new CommentReader(HippoDb::slave());
			$reader->item_type= 1; /* PROJECT */
			$reader->item_id= $project_id;
			$reader->order('added_on', 'desc');
			if (! $comments= $reader->get())
				$comments= array();
			$num_comments= count($comments);
			$this->handler_vars['total_comments']= $num_comments;
			/* Convert Bug and WL# to URLs and Codify any code blocks in comments */
			for ($i=0;$i<$num_comments;++$i)
				$comments[$i]['comment_text']= TextDecorator::convert_bugs_and_worklogs($comments[$i]['clean_text']);
			$project['comments']= $comments;


			/* Grab the rating information for this task */
			$reader= new VoteSummaryReader(HippoDb::slave());
			$reader->item_id= $project_id;
			$reader->item_type= 1; /* 1 == Project */
			if (! $project_ratings= $reader->get_one())
				$project_ratings= array();
			else
				$project_ratings['score']= number_format((int) $project_ratings['total_vote_value'] / (int) $project_ratings['total_votes'], 2);
			$project['ratings']= $project_ratings;

			if (FALSE !== $this->session->get('user_id')) {
				$reader= new VoteReader(HippoDb::slave());
				$reader->item_type= 1; /* 1 == Project */
				$reader->item_id= $project_id;
				$reader->forge_user= $this->session->get('user_id');
				if (! $vote= $reader->get_one())
					$vote= array();
			}
			else
				$vote= array();
			$project['vote']= $vote;

			$rating_units= 5;
			$rating_image_width= 30;

			$this->handler_vars['rating_image_width']= $rating_image_width;
			$this->handler_vars['rating_units']= range(1, $rating_units);
			$this->handler_vars['rating_width']= $rating_units * $rating_image_width;
			$this->handler_vars['rating_unit_count']= $rating_units;
			if (!empty($project_ratings))
				$this->handler_vars['current_vote_width']= $rating_image_width * $project_ratings['score'];

			$this->handler_vars['page_title']= 'Projects :: ' . htmlentities($project['name']);
			$this->handler_vars['breadcrumbs'][]= array('link'=>'projects/', 'title'=>'dashboard');
			$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>htmlentities($project['name']));
		}
		$this->handler_vars['project']= $project;

		/* We need to add some javascript files and CSS files for tab panel display */
		$this->handler_vars['scripts'][]= "yui-ext/adapter/yui/yui-utilities.js";
		$this->handler_vars['scripts'][]= "yui-ext/adapter/yui/ext-yui-adapter.js";

		/* If the user is logged in, pull in the AJAX scripts */
		if (FALSE !== $this->session->get('user_id')) {
			$this->handler_vars['scripts'][]= "yui-ext/adapter/prototype/prototype.js";
			$this->handler_vars['scripts'][]= "vote.js";
		}
		$this->handler_vars['scripts'][]= "yui-ext/ext-core.js";
		$this->handler_vars['scripts'][]= "yui-ext/package/util.js";
		$this->handler_vars['scripts'][]= "yui-ext/package/tabs/tabs.js";

		$this->handler_vars['stylesheets'][]= "../js/yui-ext/resources/css/ext-all.css";
		$this->handler_vars['stylesheets'][]= "../js/yui-ext/resources/css/ytheme-gray.css";

		$this->display('project');
	}

	/**
	 * Displays the home page dashboard of the public project
	 * @return void
	 * @todo Make more dynamic? 
	 */
	public function display_dashboard() {
		/* Grab a little info on the last 15 modified projects */
		$reader= new ProjectReader(HippoDb::slave());
		$reader->order('added_on', 'desc');
		/* Override default SELECT as we don't need all the info for the dashboard */
		$reader->select_sql =<<<ENDOFSQL
SELECT 
p.project_id
, p.name
, p.added_on
, p.added_by
, ab.display_name as added_by_display
FROM Project p
INNER JOIN ForgeUser ab
		ON p.added_by = ab.user_id
ENDOFSQL;
		if (! $newest_projects= $reader->get(15))
			$newest_projects= array();
		$this->handler_vars['total_projects']= $reader->get_count(); /** @todo Inefficient to get count on InnoDB tables no WHERE... */
		$this->handler_vars['newest_projects']= $newest_projects;

		/* Grab a little info on the top 10 rated projects */
		$reader= new ProjectReader(HippoDb::slave());
		/*
		 * Override default SELECT as we don't need all the info for the 
		 * dashboard and we want the ratings information too...
		 */
		$reader->select_sql =<<<ENDOFSQL
SELECT
	p.project_id
, p.name
, vs.total_votes
, FORMAT(vs.total_vote_value / vs.total_votes, 2) as rating
FROM Project p
	INNER JOIN VoteSummary vs 
		ON p.project_id = vs.item_id 
		AND vs.item_type = 1
ENDOFSQL;
		$reader->order('rating', 'desc');
		$reader->order('total_votes', 'desc');
		if (! $top_projects= $reader->get(10))
			$top_projects= array();
		else {
			$num_projects= count($top_projects);
			for ($i=0;$i<$num_projects;++$i)
				$top_projects[$i]['rating_width']= (16 * $top_projects[$i]['rating']);
		}
		$this->handler_vars['top_projects']= $top_projects;

		/* Grab the tags related to any project */
		$reader= new ItemTagReader(HippoDb::slave());
		$reader->item_type= 1; /* 1== Project */
		$reader->order('tag_text');
		if (! $project_tags= $reader->get())
			$projects_tags= array();
		$this->handler_vars['project_tags']= $project_tags;
		$this->handler_vars['tag_link']= AppConfig::get_config('DIR_HTTP') . 'projects/search.php?t=tag&k=';

		/* We need to add some javascript files for the accordian display */
		$this->handler_vars['scripts'][]= "scriptaculous/lib/prototype.js";
		$this->handler_vars['scripts'][]= "scriptaculous/src/effects.js";
		$this->handler_vars['scripts'][]= "scriptaculous/src/accordion.js";

		$this->handler_vars['page_title']= 'Projects :: Dashboard';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'dashboard');

		$this->display('dashboard');
	}

	/**
	 * Displays the form for registering a new project 
	 * @return void
	 */
	public function display_add_project() {
		/* 
		 * Grab the various lookup value lists for the 
		 * edit Project tab and add into the template
		 */
		$reader= new ProgramLanguageReader(HippoDb::slave(), $this->cache);
		$this->handler_vars['program_languages']= $reader->get_hash();

		$reader= new KnowledgeLevelReader(HippoDb::slave(), $this->cache);
		$this->handler_vars['knowledge_levels']= $reader->get_hash();

		$reader= new LicenseReader(HippoDb::slave(), $this->cache);
		$this->handler_vars['licenses']= $reader->get_hash();

		$reader= new ProjectStatusReader(HippoDb::slave(), $this->cache);
		$this->handler_vars['project_statuses']= $reader->get_hash();

		$reader= new ProjectAudienceReader(HippoDb::slave(), $this->cache);
		$this->handler_vars['project_audiences']= $reader->get_hash();

		$project= new Project(HippoDb::master());
		$project->fill_field_values($this->handler_vars);

		$this->handler_vars['page_title']= 'Projects :: Register a New Project';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'projects/', 'title'=>'dashboard');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'Register a New Project');
		$this->handler_vars['project']= $project;

		$this->display('add');
	}

	/**
	 * Displays the search form or results of search
	 * @return void
	 */
	public function display_search() {

		$this->init_search_filters();
		$search_request= new ProjectSearchRequest();

		/** 
		 * Process the search request but only if the search isn't by ID.
		 * If it is by ID, simply redirect to the Task Page for that ID
		 */
		$search_request->records_per_page= 20;

		if (!empty($this->handler_vars['ab']))
			$search_request->reader->added_by= (int) $this->handler_vars['ab'];

		if (!empty($this->handler_vars['l']))
			$search_request->reader->program_language= (int) $this->handler_vars['l'];

		if (!empty($this->handler_vars['k'])) { /* Keywords... */
			$keywords= $this->handler_vars['k'];
			$look_in= $this->handler_vars['t'];
			switch ($look_in) {
			case 'tds': /* title and description... */
				$search_request->reader->ft_match(array('pt.name', 'pt.description'), $keywords);
				break;
			case 'tag':
				$search_request->with_tags($keywords);
				break;
			}
		}
		if (! $search_results= $search_request->get())
			$search_results= array();
		else {
			$num_results= count($search_results);
			for ($i=0;$i<$num_results;++$i)
				$search_results[$i]['rating_width']= (16 * $search_results[$i]['rating']);
		}

		$this->handler_vars['search_results']= $search_results;
		$this->handler_vars['search_request']= $search_request;

		$this->handler_vars['page_title']= 'Projects :: Search Results';
		$this->handler_vars['breadcrumbs'][]= array('link'=>'project/', 'title'=>'project');
		$this->handler_vars['breadcrumbs'][]= array('link'=>'', 'title'=>'search results');

		$this->display('search-results');
	}

	/**
	 * Sets up the dropdowns for the search results filters
	 *
	 * @return void
	 */
	private function init_search_filters() {
		/* Grab the dropdown box options */
		$reader= new ProgramLanguageReader(HippoDb::slave(), $this->cache);
		if (! $program_languages= $reader->get())
			$program_languages= array();

		$tmp= array(0=>'< ALL >');
		$program_languages= $tmp + hippo_otoa($program_languages);
		$this->handler_vars['program_languages']= $program_languages;

		$reader= new ForgeUserReader(HippoDb::slave(), $this->cache);
		$reader->select_sql=<<<ENDOFSQL
SELECT DISTINCT fu.user_id, fu.display_name
FROM ForgeUser fu
INNER JOIN Project p
ON fu.user_id = p.added_by
ENDOFSQL;
		$reader->order('display_name');
		if (! $project_authors= $reader->get())
			$project_authors= array();
		$project_authors= $tmp + hippo_otoa($project_authors);
		$this->handler_vars['project_authors']= $project_authors;

		$look_ins = array();
		$look_ins['tds'] = 'Title and Description';
		$look_ins['tag'] = 'Tags';

		$this->handler_vars['look_ins']= $look_ins;

	}
}
?>
