<?php
/**
 * A specialized search request for projects
 */
class ProjectSearchRequest extends SearchRequest {
	/**
	 * Constructor
	 */
	public function __construct() {
		
		$this->result_columns[] = array('name'=>'name', 'title'=>'Name', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'added_on', 'title'=>'Added On', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'added_by_display', 'title'=>'Added By', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'program_language', 'title'=>'Language', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'rating', 'title'=>'Rating', 'is_sortable'=>true);

		$this->sort_field= 'added_on';
		$this->sort_order= 'desc';

		$reader= new ProjectReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT 
p.project_id
, p.name
, p.url
, pl.name as program_language
, p.added_on
, p.added_by
, fu.display_name as added_by_display
, vs.total_votes
, COALESCE(FORMAT(vs.total_vote_value / vs.total_votes, 2), 0) as rating
FROM Project p
	INNER JOIN ProgramLanguage pl
		ON p.program_language = pl.program_language_id 
	INNER JOIN ProjectText pt
		ON p.project_id = pt.project
	INNER JOIN ForgeUser fu
		ON p.added_by = fu.user_id
	LEFT JOIN VoteSummary vs 
		ON p.project_id = vs.item_id 
		AND vs.item_type = 1
ENDOFSQL;
		parent::__construct($reader);
	}

	/**
	 * Adds tag searching
	 *
	 * @return	void
	 * @params	tags		String of comma-separated tag phrases
	 */
	public function with_tags($tags) {
		/* First, grab a list of tag IDs for the supplied phrases */
		$reader= new TagReader(HippoDb::slave());
		$tags= explode(',', $tags);
		foreach ($tags as $tag)
			$reader->tag_text->equal_to($tag, FALSE); /* The FALSE adds an OR condition */
		if (! $tag_records= $reader->get()) 
			return FALSE;
		foreach ($tag_records as $tag_record)
			$tag_ids[]= $tag_record['tag_id'];
		$this->reader->select_sql.= " INNER JOIN Tag2Project t2p ON p.project_id = t2p.project AND t2p.tag IN (" . implode(',', $tag_ids) . ") ";
	}
}
?>
