<?php
/**
 * Generic Search Request class.
 */
class SearchRequest {
	/** Names of sortable fields */
	public $sort_fields= array();
	/** Name of field to sort by */
	public $sort_field= null;
	/** Order of sort */
	public $sort_order= 'ASC';
	/** Current page in results */
	public $current_page= 1;
	/** Last page in the results */
	public $last_page= 1;
	/** Total count of records in results */
	public $found_rows= 0;
	/** Execution time of search */
	public $execution_time= null;
	/** Records per page in results */
	public $records_per_page= 10;
	/** First record number */
	public $first_record= 0;
	/** Last record number */
	public $last_record= 0;
	/** The internal HippoReader instance that does the grunt work */
	public $reader= null;
	/** Array of information about the output results columns */
	public $result_columns= array();

	/**
	 * Constructor
	 *
	 * @return	void
	 * @param		reader	A HippoReader instance
	 */
	public function __construct($reader) {
		if (isset($_GET['page']))
			$this->current_page= (int) $_GET['page'];

		if (isset($_GET['sortby'])
			&& ! empty($_GET['sortby']))
			$this->sort_field= htmlspecialchars($_GET['sortby']);

		if (isset($_GET['sortorder'])
			&& ! empty($_GET['sortorder']))
			$this->sort_order= htmlspecialchars($_GET['sortorder']);

		if ($reader instanceof HippoReader)
			$this->reader= $reader;
		else
			trigger_error('Invalid reader instance supplied.', E_USER_ERROR);
	}

	/** 
	 * Returns the search results
	 *
	 * @return	array		The results of search
	 */
	public function get() {
		/* Sanity Checks */
		if ((int) $this->current_page < 1)
			$this->current_page= 1;
        
		if (!empty($this->sort_field))
			$this->reader->order($this->sort_field, $this->sort_order);
        
		/*
		 * Here, we check that the reader hasn't "gone past" the 
		 * end of the reader before we actually issue the limit 
		 * query...
		 */
    $this->found_rows= $this->reader->get_count();
		if ($this->current_page > ceil($this->found_rows / $this->records_per_page)) {
			$this->current_page= ceil($this->found_rows / $this->records_per_page);
		}

		$offset= max((($this->current_page - 1) * $this->records_per_page), 0);

		$this->reader->limit($this->records_per_page, $offset);
		$results= $this->reader->get();
    $this->found_rows= $this->reader->get_count();
		if ($this->records_per_page >= $this->found_rows)
			$this->last_page= 1;
		else
			$this->last_page= ceil($this->found_rows/$this->records_per_page);

		/* Calculate the first and last record "numbers" */
		if ($this->current_page == 1) {
			$this->first_record= 1;
			if ($this->found_rows < $this->records_per_page)
				$this->last_record= $this->found_rows;
			else
				$this->last_record= $this->records_per_page;
		}
		elseif ($this->current_page == $this->last_page) {
			$this->first_record= ($this->records_per_page * ($this->current_page - 1)) + 1;
			$this->last_record= $this->found_rows;
		}
		else {
			$this->first_record= ($this->records_per_page * ($this->current_page - 1)) + 1;
			$this->last_record= ($this->records_per_page * ($this->current_page));
		}

		return $results;
        
  }    
}
?>
