<?php
/**
 * Class for cleaning and purifying text.  Makes heavy use of the 
 * HTMLPurifier library
 */
class TextCleaner {
  /** Instance of the HTML Purifier  */
  protected static $html_purifier= null;

	/**
	 * Checks to see if the purifier is instantiated.  If not,
	 * instantiates it and returns it.
	 *
	 * @return	instance of HTMLPurifier
	 */
	protected static function purifier() {
		if (! is_null(self::$html_purifier))
			return self::$html_purifier;

		/* 
		 * Configure HTMLPurifier here for safe checks with some allowed 
		 * tags for ease of use.
		 */
		include(APP_DIR . '/opt/htmlpurifier/library/HTMLPurifier.auto.php');
		self::$html_purifier= new HTMLPurifier();
		return self::$html_purifier;
	}

	/**
	 * Cleans the subject text to contain only harmless HTML elements
	 * as well as normalize newlines into <p> tags and ensure XHTML 1.0 Trans
	 * compliance
	 *
	 * @return 	string	The cleaned text
	 * @param		subject	The text to clean
	 */
	public static function clean_html($subject) {
		if (! empty($subject) && !is_string($subject)) {
			throw new Exception('$subject passed to TextCleaner::clean_html() was not a string');
		}
		$purifier= TextCleaner::purifier();
		$config= HTMLPurifier_Config::createDefault();
		$config->set('HTML', 'Doctype', 'XHTML 1.0 Transitional');
		$config->set('HTML', 'AllowedElements', 'a,em,blockquote,p,code,pre,strong,b');
		$config->set('HTML', 'AllowedAttributes', 'a.href,a.title');
		$config->set('AutoFormat', 'AutoParagraph', true); /* Normalize Newlines */
		$subject= $purifier->purify($subject, $config);
		return $subject;
	}	
}
?> 
