<?php
/**
 * A specialized search request for worklog tasks
 */
class ToolSearchRequest extends SearchRequest {
	/**
	 * Constructor
	 */
	public function __construct() {
		
		$this->result_columns[] = array('name'=>'title', 'title'=>'Name/Title', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'tool_type', 'title'=>'Type', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'added_on', 'title'=>'Added On', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'added_by_display', 'title'=>'Author', 'is_sortable'=>true);
		$this->result_columns[] = array('name'=>'rating', 'title'=>'Rating', 'is_sortable'=>true);

		$this->sort_field= 'added_on';
		$this->sort_order= 'desc';

		/** @todo Refactor to remove encapsulation here...loose couple it through constructor.*/
		$reader= new ToolReader(HippoDb::slave());
		$reader->select_sql=<<<ENDOFSQL
SELECT SQL_NO_CACHE
	t.tool_id
, t.title
, ab.display_name as added_by_display
, t.added_by
, t.added_on
, tt.name as tool_type
, vs.total_votes
, COALESCE(FORMAT(vs.total_vote_value / vs.total_votes, 2), 0) as rating
FROM Tools t 
	INNER JOIN ToolTypes tt
		ON t.tool_type = tt.type_id
	INNER JOIN ForgeUser ab
		ON t.added_by = ab.user_id
	LEFT JOIN VoteSummary vs 
		ON t.tool_id = vs.item_id 
		AND vs.item_type = 2
ENDOFSQL;
		parent::__construct($reader);
	}

	/**
	 * Adds tag searching
	 *
	 * @return	void
	 * @params	tags		String of comma-separated tag phrases
	 */
	public function with_tags($tags) {
		/* First, grab a list of tag IDs for the supplied phrases */
		$reader= new TagReader(HippoDb::slave());
		$tags= explode(',', $tags);
		foreach ($tags as $tag)
			$reader->tag_text->equal_to($tag, FALSE); /* The FALSE adds an OR condition */
		if (! $tag_records= $reader->get()) 
			return FALSE;
		foreach ($tag_records as $tag_record)
			$tag_ids[]= $tag_record['tag_id'];
		$this->reader->select_sql.=<<<ENDOFSQL

	INNER JOIN Tag2Tool t2w 
		ON t.tool_id = t2w.tool 
		AND t2w.tag IN (
ENDOFSQL;
		$this->reader->select_sql.= implode(',', $tag_ids) . ") ";
	}
}
?>
