<?php
/**
 * The TrackingHandler is the handler which handles all actions
 * related to "watching" or "observing" an item (project, task, tool, etc)
 */
class TrackingHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= '';
		$this->handler_vars['CUR_NAV']= 'home';
		parent::__construct();
		/* All voting functionality requires user to be logged in */
		if (FALSE === $this->session->get('user_id'))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . 'denied.php');
	}

	/**
	 * Helper function for returning or redirecting...
	 *
	 * @return	mixed		Either JSON data or a redirect occurs.
	 */
	private function dispatch($success, $msg) {
		if (! function_exists('json_encode')) {
			/** @todo Add non 5.2 json_encode() */
			function json_encode() {}
		}
		if (isset($this->handler_vars['json'])) {
			header("Content-type: application/json");
			echo '{"results":' . json_encode(array('success'=>(bool) $success, 'msg'=>$msg)) . '}';
			exit;
		}
		else {
			/* We have to pass the message via the session */
			if (!$success) {
				$errors= array('observer'=>$msg);
				$this->session->set('errors', $errors);
			}
			HippoHttpResponse::redirect($_SERVER['HTTP_REFERER']);
		}
	}

	/**
	 * Toggles tracking (watching) a specified item and returns JSON-encoded
	 * success or failure.
	 *
	 * @return	string	JSON encoded message
	 * @todo		Make this less coupled to SQL.  Use HippoRecords.
	 */
	public function json_track() {
		$item_type_map= array('project'=>1, 'tool'=>2, 'worklog'=>3);
		if (isset($this->handler_vars['item'])) {
			$item= $this->handler_vars['item'];
			$item_type= $item_type_map[$item];
		}
		else
			$this->errors[]= 'No item specified';

		if (isset($this->handler_vars['item_id']))
			$item_id= (int) $this->handler_vars['item_id'];
		else
			$this->errors[]= 'No ID specified';

		$user_id= (int) $this->session->get('user_id');

		if (empty($this->errors)) {
			switch (strtolower($item)) {
			case 'worklog':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "worklog/task.php?id={$item_id}";
				break;
			case 'project':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$item_id}";
				break;
			case 'tool':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "tools/tool.php?id={$item_id}";
				break;
			case 'forge_user':
				$url_redirect= AppConfig::get_config('DIR_HTTP') . "projects/project.php?id={$project_id}";
				break;
			}

			$reader= new ObserverReader(HippoDb::master());
			$reader->item_type= (int) $item_type;
			$reader->item_id= (int) $item_id;
			$reader->forge_user= (int) $user_id;

			$add= TRUE;
			if ($reader->get_count() == 1)
				$add= FALSE;

			$observer= new Observer(HippoDb::master());
			$observer->item_type= (int) $item_type;
			$observer->item_id= (int) $item_id;
			$observer->forge_user= (int) $user_id;

			if ($add && $observer->save()) {
				//HippoEvents::notify("on_after_{strtolower($item)}_add_observer", $observer);
				//$this->dispatch(TRUE, "Watch on {$item} initiated.");
			}
			elseif ((! $add) && $observer->remove()) {
				//$this->dispatch(TRUE, "Watch on {$item} removed.");
				//HippoEvents::notify("on_after_{strtolower($item)}_remove_observer", $observer);
			}
			else {
				$this->errors['observer']= "Error saving watch: " . HippoDb::master()->get_last_error_message();
			}
			HippoHttpResponse::redirect($url_redirect);
		}
		else {
			print_r($this->errors); //HippoHttpResponse::redirect($SERVER['HTTP_REFERER']);
			return '{"results":' . json_encode(array('success'=>FALSE, 'msg'=>$this->errors[0])) . '}';
		}
	}
}
?>

