<?php
/**
 * The VoteHandler is the handler which handles all actions
 * related to the unified voting system
 */
class VoteHandler extends ForgeUIHandler {
	public function __construct() {
		$this->template_subdir= '';
		$this->handler_vars['CUR_NAV']= 'home';
		parent::__construct();
		/* All voting functionality requires user to be logged in */
		if (FALSE === $this->session->get('user_id'))
			HippoHttpResponse::redirect(AppConfig::get_config('DIR_HTTP') . 'denied.php');
	}

	/**
	 * Helper function for returning or redirecting...
	 *
	 * @return	mixed		Either JSON data or a redirect occurs.
	 */
	private function dispatch($success, $msg) {
		if (isset($this->handler_vars['json'])) {
			header("Content-type: application/json");
			/* 
			 * Return the vote counts and values for the item and id
			 * in question so that the page element can update itself
			 */
			$item_type_map= array('project'=>1, 'tool'=>2, 'worklog'=>3);
			if (isset($this->handler_vars['item'])) {
				$item= $this->handler_vars['item'];
				$item_type= $item_type_map[$item];
			}
			else
				$this->errors[]= 'No item specified';

			if (isset($this->handler_vars['id']))
				$item_id= (int) $this->handler_vars['id'];
			else
				$this->errors[]= 'No ID specified';

			$reader= new VoteSummaryReader(HippoDb::slave());
			$reader->item_type= (int) $item_type;
			$reader->item_id= (int) $item_id;
			$ratings= $reader->get_one();
			$ratings['score']= number_format((int) $ratings['total_vote_value'] / (int) $ratings['total_votes'], 2);

			$rating_units= 5;
			$rating_image_width= 30;
			$ratings['current_vote_width']= $rating_image_width * $ratings['score'];
			$ratings['unit_count']= $rating_units;

			echo '{"results":' . json_encode(array('success'=>(bool) $success, 'ratings'=>$ratings)) . '}';
			exit;
		}
		else {
			/* We have to pass the message via the session */
			if (!$success) {
				$this->errors= array('vote'=>$msg);
			}
			HippoHttpResponse::redirect($_SERVER['HTTP_REFERER']);
		}
	}

	/**
	 * Inserts a vote for a specified item and returns JSON-encoded
	 * success or failure.
	 *
	 * @return	string	JSON encoded message
	 */
	public function json_vote() {
		$item_type_map= array('project'=>1, 'tool'=>2, 'worklog'=>3);
		if (isset($this->handler_vars['item'])) {
			$item= $this->handler_vars['item'];
			$item_type= $item_type_map[$item];
		}
		else
			$this->errors[]= 'No item specified';

		if (isset($this->handler_vars['id']))
			$item_id= (int) $this->handler_vars['id'];
		else
			$this->errors[]= 'No ID specified';

		if (isset($this->handler_vars['vote']))
			$vote_value= $this->handler_vars['vote'];
		else
			$this->errors[]= 'No vote specified';

		if (empty($this->errors)) {
			/*
			 * We grab the previous vote, if any, so that
			 * we can pre-calculate the increment or decrement
			 * of the vote summary value
			 */
			$reader= new VoteReader(HippoDb::slave());
			$reader->item_type= (int) $item_type;
			$reader->item_id= (int) $item_id;
			$reader->forge_user= (int) $this->session->get('user_id');
			if (! $previous_vote= $reader->get_one())
				$previous_vote= array();

			if (! empty($previous_vote)) {
				$previous_vote_value= (int) $previous_vote['vote_value'];
				$vote_value_adjustment= ($vote_value - $previous_vote_value);
				$total_vote_adjustment= 0;
			}
			else {
				$vote_value_adjustment= $vote_value;
				$total_vote_adjustment= 1;
			}

			$connection= HippoDb::master();
			$vote= new Vote($connection);
			$vote->item_type= (int) $item_type;
			$vote->item_id= (int) $item_id;
			$vote->forge_user= (int) $this->session->get('user_id');
			$vote->vote_value= (int) $vote_value;
			$vote->voted_on= date('Y-m-d H:i:s');

			$connection->start_transaction();
			if ($vote->save()) {
				/* Update the VoteSummary as well... */
				$sql=<<<ENDOFSQL
INSERT INTO VoteSummary (item_type, item_id, total_votes, total_vote_value) 
VALUES (?,?,1,?)
ON DUPLICATE KEY UPDATE total_votes=total_votes+{$total_vote_adjustment}, total_vote_value=(total_vote_value+({$vote_value_adjustment}))
ENDOFSQL;
				$connection->add_param($item_type);
				$connection->add_param($item_id);
				$connection->add_param($vote_value);
				if ($connection->query($sql)) {
					$connection->commit();
					$this->dispatch(TRUE, 'Vote succeeded');
				}
				else  {
			print_r($connection);exit;
					$connection->rollback();
					$this->dispatch(FALSE, 'Ratings update failed: ' . $connection->get_last_error_message());
				}
			}
			else  {
				$connection->rollback();
				return '{"results":' . json_encode(array('success'=>FALSE, 'msg'=>$vote->get_errors())) . '}';
			}
		}
		else  {
			$connection->rollback();
			print_r($this);
			return '{"results":' . json_encode(array('success'=>FALSE, 'msg'=>$this->errors[0])) . '}';
		}
	}
}
?>
