CREATE TABLE `knowledge_levels` (
  `knowledge_level_id` tinyint(3) unsigned NOT NULL auto_increment,
  `description` varchar(30) NOT NULL default '',
  PRIMARY KEY  (`knowledge_level_id`)
) ENGINE=InnoDB AUTO_INCREMENT=4 DEFAULT CHARSET=utf8;

INSERT INTO `knowledge_levels` VALUES (1,'Novice'),(2,'Intermediate'),(3,'Expert');

CREATE TABLE `licenses` (
  `license_id` tinyint(3) unsigned NOT NULL auto_increment,
  `name` varchar(64) NOT NULL default '',
  PRIMARY KEY  (`license_id`)
) ENGINE=InnoDB AUTO_INCREMENT=56 DEFAULT CHARSET=utf8;

INSERT INTO `licenses` 
VALUES (1,'The GNU General Public License (GPL)'),(2,'The GNU Library or \"Lesser\" Public License (LGPL)'),(3,'The BSD license'),
(4,'The MIT license'),(5,'The Artistic license'),(6,'The Mozilla Public License v. 1.0 (MPL)'),(7,'The Qt Public License (QPL)'),
(8,'The IBM Public License'),(9,'The MITRE Collaborative Virtual Workspace License (CVW License)'),(10,'The Ricoh Source Code Public License'),
(11,'The Python license (CNRI Python License)'),(12,'The Python Software Foundation License'),(13,'The zlib/libpng license'),(14,'The Apache Software License'),
(15,'The Vovida Software License v. 1.0'),(16,'The Sun Industry Standards Source License (SISSL)'),(17,'The Intel Open Source License'),
(18,'The Mozilla Public License 1.1 (MPL 1.1)'),(19,'The Jabber Open Source License'),(20,'The Nokia Open Source License'),(21,'The Sleepycat License'),
(22,'The Nethack General Public License'),(23,'The Common Public License'),(24,'The Apple Public Source License'),(25,'The X.Net License'),
(26,'The Sun Public License'),(27,'The Eiffel Forum License'),(28,'The W3C License'),(29,'The Motosoto License'),(30,'The Open Group Test Suite License'),
(31,'OSI Approved (Open Source)'),(32,'Public Domain/Freeware'),(33,'GNU Free Documentation License (FDL)'),(34,'The PHP License'),(35,'The OpenLDAP Public License'),
(36,'The SUN Community Source License'),(37,'The Zope Public License (ZPL)'),(38,'The Clarified Artistic License'),(39,'The Voxel Public License (VPL)'),
(40,'The Netscape Public License (NPL)'),(41,'The SUN Binary Code License'),(42,'The Latex Project Public License (LPPL)'),(43,'The Open Content License'),
(44,'The Open Public License'),(45,'The Open Publication License'),(46,'The Open Public License'),(47,'The Aladdin Free Public License (AFPL)'),
(48,'The Arphic Public License'),(49,'FreeType License'),(50,'The Interbase Public License'),(51,'The Phorum License'),(52,'The Plan 9 Open Source License'),
(53,'Non-OSS Commercial License'),(54,'Other - see project for details'),(55,'Multiple - see project for details');

CREATE TABLE `program_languages` (
  `program_language_id` smallint(5) unsigned NOT NULL auto_increment,
  `name` varchar(30) NOT NULL default '',
  PRIMARY KEY  (`program_language_id`),
  UNIQUE KEY `iux_name` (`name`)
) ENGINE=InnoDB AUTO_INCREMENT=30 DEFAULT CHARSET=utf8;

INSERT INTO `program_languages` 
VALUES (2,'ASP'),(3,'ASP.NET'),(4,'Assembly'),(1,'AWK'),(24,'bash'),(5,'C'),(7,'C#'),(6,'C++'),(8,'Delphi/Kylix'),(9,'Eiffel'),(10,'Java'),(11,'JavaScript'),
(12,'Lisp'),(29,'Lua'),(27,'None/Text'),(13,'Objective C'),(28,'Other'),(14,'Pascal'),(15,'Perl'),(16,'PHP'),(18,'PROGRESS'),(19,'Python'),(20,'Ruby'),
(21,'Scheme'),(22,'SmallTalk'),(17,'SQL'),(23,'Tcl'),(26,'VB.NET'),(25,'VB6');

CREATE TABLE `project_audiences` (
  `project_audience_id` tinyint(3) unsigned NOT NULL auto_increment,
  `description` varchar(30) NOT NULL default '',
  PRIMARY KEY  (`project_audience_id`)
) ENGINE=InnoDB AUTO_INCREMENT=10 DEFAULT CHARSET=utf8;

INSERT INTO `project_audiences` 
VALUES (1,'Software Developers/Engineers'),(2,'System Administrators'),(3,'Database Administrators'),
(4,'Network Administrators'),(5,'Web Designers'),(6,'Web Developers'),(7,'Project Managers'),(8,'End Users/Desktop Users'),(9,'General');

CREATE TABLE `project_statuses` (
  `project_status_id` tinyint(3) unsigned NOT NULL default '0',
  `description` varchar(25) NOT NULL default '',
  PRIMARY KEY  (`project_status_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO `project_statuses` VALUES (1,'Early (Pre-Alpha)'),(2,'Development - Alpha'),(3,'Development - Beta'),(4,'Production');

CREATE TABLE `architectures` (
  `architecture_id` tinyint(3) unsigned NOT NULL auto_increment,
  `name` varchar(60) NOT NULL default '',
  PRIMARY KEY  (`architecture_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

INSERT INTO `architectures` 
VALUES 
(1,'x86')
,(2,'x86_64')
,(3,'AMD')
,(4,'AMD 64')
,(5,'PowerPC')
,(6,'PowerPC64')
,(7,'Solaris')
,(8,'Alpha')
,(9,'IA64')
,(10,'MIPS')
,(11,'ARM')
,(12,'HPPA')
,(13,'PPCMacOS')
,(14,'s390')
,(15,'Sparc');

CREATE TABLE `projects` (
  `project_id` int(10) unsigned NOT NULL auto_increment,
  `name` varchar(150) NOT NULL default '',
  `url` varchar(200) NOT NULL default '',
  `license_id` tinyint(3) unsigned NOT NULL default '0',
  `primary_status_id` tinyint(3) unsigned NOT NULL default '0',
  `target_audience_id` tinyint(3) unsigned NOT NULL default '0',
  `target_knowledge_level_id` tinyint(3) unsigned NOT NULL default '0',
  `program_language_id` smallint(5) unsigned NOT NULL default '0',
  `added_by` int(10) unsigned NOT NULL default '0',
  `added_on` datetime NOT NULL default '0000-00-00 00:00:00',
  `last_updated_by` int(10) unsigned NOT NULL default '0',
  `last_updated_on` datetime NOT NULL default '0000-00-00 00:00:00',
  PRIMARY KEY  (`project_id`),
  UNIQUE KEY `uix_name` (`name`(45)),
  KEY `fk_added_by` (`added_by`),
  CONSTRAINT `fk_licenses` FOREIGN KEY (`license_id`) REFERENCES `licenses` (`license_id`),
  CONSTRAINT `fk_project_statuses` FOREIGN KEY (`primary_status_id`) REFERENCES `project_statuses` (`project_status_id`),
  CONSTRAINT `fk_project_audiences` FOREIGN KEY (`target_audience_id`) REFERENCES `project_audiences` (`project_audience_id`),
  CONSTRAINT `fk_knowledge_levels` FOREIGN KEY (`target_knowledge_level_id`) REFERENCES `knowledge_levels` (`knowledge_level_id`),
  CONSTRAINT `fk_program_languages` FOREIGN KEY (`program_language_id`) REFERENCES `program_languages` (`program_language_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

CREATE TABLE `project_text` (
  `project_id` int(10) unsigned NOT NULL default '0',
  `description` text NOT NULL,
  `name` varchar(150) NOT NULL,
  PRIMARY KEY  (`project`),
  FULLTEXT KEY `ft_description_name` (`name`,`description`)
) ENGINE=MyISAM DEFAULT CHARSET=utf8;
