<?php
/**
 * Module file for the project directory application
 */
class ProjectsModule extends ForgeModule {

	/**
	 * Initializes the module.  Use this method to add
	 * event listeners, append to ForgeUrl, etc...
	 */
	public function init() {

		/* Add any rewriters */
		$rules= array();
		$rule= new ForgeUrlRule('projects_dashboard');
		$rule->set_inbound_match("projects")->set_inbound_callback(array($this, 'dashboard'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('projects_add');
		$rule->set_inbound_match("projects/add")->set_inbound_callback(array($this, 'add_person'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('projects_delete');
		$rule->set_inbound_match("projects/delete")->set_inbound_callback(array($this, 'delete_person'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('projects_edit');
		$rule->set_inbound_match("projects/edit")->set_inbound_callback(array($this, 'edit_person'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('projects_view_by_id');
		$rule->set_inbound_match("/^projects\/view\/(\d+)$/i")
			->set_inbound_match_vars(array('user_id'))
			->set_inbound_callback(array($this, 'view_person_by_id'));
		$rules[$rule->key]= $rule;

		$rule= new ForgeUrlRule('projects_view_by_slug');
		$rule->set_inbound_match("/^projects\/view\/([[:alpha:]]+)$/i")
			->set_inbound_match_vars(array('slug'))
			->set_inbound_callback(array($this, 'view_person_by_display_slug'));
		$rules[$rule->key]= $rule;

		ForgeUrl::append_rules($rules);
		
		HippoEvents::listen('on_before_display_template', array($this, 'add_menu_items'));
		HippoEvents::listen('on_before_display_dashboard', array($this, 'add_dashboard_items'));

	}

	/**
	 * Adds menu items for display in the layout template
	 */
	public function add_menu_items() {
		$controller= $this->controller;
		
		$main_menu_items= array();
		$main_menu_items[]= array(
			'title'=>ForgeTranslate::get('projects')
			, 'link'=>ForgeUrl::out('projects_dashboard')
		);

		/* Add some submenu items if we're actually in this module... */
		if ($controller->active_module == __CLASS__) {
			$submenu_items= array();
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('home')
				, 'link'=>ForgeUrl::out('projects_dashboard')
			);
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('browse projects')
				, 'link'=>ForgeUrl::out('projects_search')
			);
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('newest projects')
				, 'link'=>ForgeUrl::out('projects_search', array('sortby'=>'added_on','sortorder'=>'desc'))
			);
			$submenu_items[]= array(
					'title'=>ForgeTranslate::get('top-rated projects')
				, 'link'=>ForgeUrl::out('projects_search', array('sortby'=>'rating','sortorder'=>'desc'))
			);
			if ($controller->user_logged_in) {
				$submenu_items[]= array(
						'title'=>ForgeTranslate::get('register a project')
					, 'link'=>ForgeUrl::out('projects_add')
					, 'special'=>TRUE
				);
			}
			$controller->handler->append('submenu_items', $submenu_items);
			$main_menu_items[0]['active']= TRUE;
		}
		$controller->handler->append('menu_items', $main_menu_items);

	}

	/**
	 * Adds newest members to the home dashboard
	 */
	public function add_dashboard_items() {
		$controller= $this->controller;

		/* Grab a little info on the newest few people */
		$reader= new ProjectReader(HippoDb::slave());
		/* Override default SELECT as we don't need all the info for the dashboard */
		$reader->select_sql =<<<ENDOFSQL
SELECT
	p.project_id
, p.name as title
, p.added_on
FROM Project p
ENDOFSQL;
		$reader->order('added_on', 'desc');
		if (! $new_items= $reader->get(ForgeConfig::get('NUM_ITEMS_DASHBOARD')))
			$new_items= array();
		else {
			$num_items= count($new_items);
			for ($i=0;$i<$num_items;++$i)
				$new_items[$i]['link']= ForgeUrl::out('project_view', $new_items[$i]['project_id']);
		}
		$controller->handler->append('new_items', $new_items);
	}
}
?>
