<?php
/*
 * Adds and removes tags to/from the project
 */
require_once('../init.php');

$user_id = $GLOBALS['WebSession']->Get('user_id');
if (empty($user_id)) {
    redirect(DIR_HTTPS_BASE . 'myforge/');
}

$project_id = $GLOBALS['Controller']->Get('id');
$tag_text = $GLOBALS['Controller']->Get('tag_text');
$deleted_tags = $GLOBALS['Controller']->Get('deleted_tags');
$for_review = false;

include(DIR_LIB . 'class/Project.php');
$project =& new Project();
$project->Id = $project_id; // No need to load here... just populate the ID...

include(DIR_LIB . 'class/ProjectStat.php');
$stat =& new ProjectStat();

if (empty($deleted_tags) && empty($tag_text)) {
    redirect(DIR_HTTP_BASE . 'projects/view.php?id=' . $project_id);    
}

if (!empty($deleted_tags)) {

    /**
     * Handles removal of tags first
     */
    $num_deleted_tags = count($deleted_tags);
    if ($num_deleted_tags > 0) {
        for ($i=0;$i<$num_deleted_tags;++$i) {
            $project->RemoveTag($deleted_tags[$i]);
        }
        $stat->Log('tag', $project_id, $Increment=false, $CounterValue=$num_deleted_tags);
    }
}

if (!empty($tag_text)) {

    /** 
     * Split the tags via comma-delimiters, then remove
     * any special characters and normalize
     */
    $tags = explode(',', $tag_text);
    $tags = array_map('trim', $tags);
    $tags = array_unique($tags);
    $num_new_tags = count($tags);

    /* Strip out the crap */
    $tmp_tags = array();
    for ($i=0;$i<$num_new_tags;++$i) {
        if ($tags[$i] == '') {continue;}
        $tmp_tags[] = tagify($tags[$i]);
    }
    $tags = $tmp_tags;
    $num_new_tags = count($tags);
    unset($tmp_tags);

    /**
     * Check for any similar matches in existing
     * tag base.  If found, separate them out and
     * present to user for approval
     */
    include(DIR_LIB . 'class/TagFinder.php');
    $finder =& new TagFinder();

    /**
     * First, strip out tags matching identically, since
     * no need to match similar tags
     */
    $exact_tags = $finder->GetExactByText($tags);
    $exact_tag_texts = array_map('strtolower', get_column_from_array($exact_tags, 'tag_text'));

    $num_exact_tags = count($exact_tags);
    if ($num_exact_tags > 0) {
        $tmp_tags = array();
        for ($i=0;$i<$num_new_tags;++$i) {
            if (!in_array(strtolower($tags[$i]), $exact_tag_texts)) {
                $tmp_tags[] = $tags[$i];
            }
        }
        $tags = $tmp_tags;
        unset($tmp_tags, $exact_tag_texts);
    }

    $review_tags = array(); /* Keyed based on original tag text */
    $new_tags = array(); /* Holds brand new tags, with no similar or exact matching */
    $num_new_tags = count($tags);
    for ($i=0;$i<$num_new_tags;++$i) {

        $current_tag_text = $tags[$i];
        /* Only bother with words more than 2 characters in length */
        if (strlen($current_tag_text) < 3) {
            $new_tags[] = $current_tag_text;
            continue;
        }

        $matched_tags = $finder->GetSimilarByText($current_tag_text);
        if (count($matched_tags) > 0) {
            $review_tags[$current_tag_text] = $matched_tags;
        }
        else {
            $new_tags[] = $current_tag_text;
        }
    }

    $num_new_tags = count($new_tags);
    $for_review  = (count($review_tags) > 0);

    /**
     * OK, let's go ahead and add the exact match tags now
     * since no interaction is required for those
     */
    if ($num_exact_tags > 0) {
        for ($i=0;$i<$num_exact_tags;++$i) {
            if ($project->AddTag($exact_tags[$i]['tag_text'])) {
                $stat->Log('tag', $project_id);
            }
        }
    }

    /**
     * OK, now let's add the non-matching tag texts
     */
    if ($num_new_tags > 0) {
        for ($i=0;$i<$num_new_tags;++$i) {
            if ($project->AddTag($new_tags[$i])) {
                $stat->Log('tag', $project_id);
            }
        }
    }
}

if ($for_review) {
    $GLOBALS['TEngine']->AssignByRef('ReviewTags', $review_tags);
    $GLOBALS['TEngine']->AssignByRef('ProjectId', $project_id);
    $GLOBALS['TEngine']->Assign('ContentTemplate', DIR_TEMPLATES . 'projects/review-tags.tpl');
    $GLOBALS['TEngine']->Assign('PageTitle', SITE_NAME . ' :: View Project :: Review Tags');
    $GLOBALS['TEngine']->SetTemplate(DIR_TEMPLATES . 'layout.tpl');
    $GLOBALS['TEngine']->Display();
}
else {
    redirect(DIR_HTTP_BASE . 'projects/view.php?id=' . $project_id);
}
?>